// *******************************************************
//
//   Program:  P1_Main.c        Version: 1.1 
//  
//   Author:  Stephen Swift		Date:  Oct 20, 2004
//
//   Copyright 2004, SOC Machines, Inc. 
//
//   Main definition include file for P1 Blackfin Monitor
//
// *******************************************************
#ifndef  __P1_FLAGS_DEFINED
#define __P1_FLAGS_DEFINED

//--------------------------------------------------------------------------//
// Header files																//
//--------------------------------------------------------------------------//
#include <sys\exception.h>
#include <cdefBF533.h>

//--------------------------------------------------------------------------//
// Symbolic constants														//
//--------------------------------------------------------------------------//

/* USART Buffer Defines */
#define UART_RX_BUFFER_SIZE 64 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#define UART_TX_BUFFER_SIZE 64 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )
#define 	NO  		  0x00
#define 	YES 		  0x01
#define 	ON  		  0x00
#define 	OFF 		  0x01
#define 	XON   		  0x11
#define 	XOFF  		  0x13d
#define  	SPACE 	   	  0x20 

// Video Global Definitions
// Start address of YUV Buffer filled by DMA from CM64
#define YUVBufPtr   0x00000000
#define pYUVBufPtr ((volatile unsigned char *)YUVBufPtr)
// Start address of RGB Buffer - 2,097,152 byte offset from 0
#define RGBBufPtr   0x00200000
#define pRGBBufPtr ((volatile unsigned char *)RGBBufPtr)

// GLobal Address Definitions
#define SDRAMBot   0x00000000
#define pSDRAMBot ((volatile unsigned char *)SDRAMBot)
#define SDRAMWordBot   0x00000000
#define pSDRAMWordBot ((volatile unsigned int *)SDRAMWordBot)
#define SDRAMTop   0x01FFFFFF
#define pSDRAMTop ((volatile unsigned char *)SDRAMTop)


//  EEPROM Definitions
#define   MAJORVER   	 0
#define   MINORVER       52
#define   BPROCESSORTYPE400  70  // P1 processor - 400MHz
#define   BPROCESSORTYPE600X 71  // P1 processor - 600MHz X Version
#define   BPROCESSORTYPE600  72  // P1 processor - 600MHz
#define   BPROCESSORTYPE7500 72  // P1 processor - 600MHz
#define   AVRPROCESSORTYPE16 10  // P1 AVR 16K Flash, 8MHz
#define   AVRPROCESSORTYPE32 12  // P1 AVR 32K Flash, 8MHz
#define   FLASHTYPE512K	 10		 // P1 Flash 512Kx8
#define	  FLASHTYPE1M	 11		 // P1 Flash 1Mx8
#define   FLASHTYPE2M	 12		 // P1 Flash 2Mx8
#define   SDRAMTYPE4M	 12		 // P1 SDRAM 4Mx16
#define   SDRAMTYPE8M	 12		 // P1 SDRAM 8Mx16
#define   SDRAMTYPE16M	 12		 // P1 SDRAM 16Mx16
#define   ETHERNETMACID  "004565000000"
#define   IPADDRESS      "255.128.0.24"
#define   IPMASK         "255.255.255.0"
#define   IPGATEWAY		 "255.45.25.0"
#define   USERID		 "PIRANA1"
#define   PASSWORD       "SECRET"

// Other definitions
#define  LEEPROM   		'e'
#define  LBlackfinSDRAM 'a'
#define  LFLASH    		'f'
#define  BOOTMODE0 		'0'
#define  BOOTMODE2 		'2'
#define  BOOTMODE3 		'3'
#define  READCONTINUOUS  1
#define  READBUFFER      2
#define  CR   			0x0D
#define  LF   			0x0A
#define  INTELHEXBYTESPERLINE  16
#define  PRINTINTELHEXASCII 1


//--------------------------------------------------------------------------//
// Global variables															//
//--------------------------------------------------------------------------//

//--------------------------------------------------------------------------//
// Prototypes																//
//--------------------------------------------------------------------------//
void Init_Flags(void);
void Init_Timers(void);
void Init_EBIU(void);
void Init_Flash(void);
void Init_Interrupts(void);
void YUVtoRGB(int);
void RGBtoYUV(void);

// ISRs Definitions
EX_INTERRUPT_HANDLER(Timer0_ISR);
//	EX_INTERRUPT_HANDLER(DMA0_PPI_ISR);	// assign DMA0 PPI ISR to interrupt vector 8
EX_INTERRUPT_HANDLER(UART_RX_ISR);		// UART Rx ISR -> IVG 9	
EX_INTERRUPT_HANDLER(UART_TX_ISR);
#endif //__P1_FLAGS_DEFINED

