// **************************************************************************
//
//  File:  P1_Utilities.c				Version: 0.67
//
//  Date:  October 20, 2004		
//
//  Utilities code for P1 Blackfin Monitor
//
// **************************************************************************
/*
 * Copyright (C) 2005-2006 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_Utilities.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
#include "P1_Main.h"
#include "ccblkfn.h"
#include "sysreg.h"

#include "P1_I2C.h"
#include "SAA7113.h"  
#include "math.h"
#include "P1_Extern.h"

//****************************************************************
//****************************************************************

//  Delay for the selected number of microseconds
//    A 130.5MHz SCLK counts 118.6 ticks per microsecond
//    Max delay is 36.2 seconds
void Delay(long microseconds)
{
	long countperiod;
	
	countperiod = microseconds * 131;
	//  Use timer 1 as count up timer
	*pTIMER1_CONFIG	= 0x0049;
	*pTIMER1_PERIOD	= countperiod;
	*pTIMER1_WIDTH = 0x00000006;
	*pTIMER_ENABLE |= 0x0002;
	// Timer stops once period count is reached
	*pTIMER_DISABLE |= 0x0002;
	
	while(*pTIMER1_COUNTER < countperiod) ;
	*pTIMER_DISABLE &= 0x000D;	
	*pTIMER_ENABLE &= 0x000D; 
}

//  Delay for the selected number of nanoseconds
//    A 118.8MHz SCLK counts 118.6 ticks per microsecond
//    or Max delay is 36.2 seconds
void Delaynano(long nanoseconds)
{
	long countperiod;
	
	countperiod = nanoseconds*8;
	//  Use timer 1 as count up timer
	*pTIMER1_CONFIG	= 0x0049;
	*pTIMER1_PERIOD	= countperiod;
	*pTIMER1_WIDTH = 0x00000006;
	*pTIMER_ENABLE |= 0x0002;
	// Timer stops once period count is reached
	*pTIMER_DISABLE |= 0x0002;
	
	while(*pTIMER1_COUNTER < countperiod) ;
	*pTIMER_DISABLE &= 0x000D;	
	*pTIMER_ENABLE &= 0x000D; 
}

// Convert ASCII hex to binary
unsigned char hextobyte(unsigned char *instrg)
{
 	 unsigned char lownibble,highnibble;
	 
 	 //  Convert high byte
	 if((instrg[0] >= 'A')&(instrg[0]<='F')) highnibble = instrg[0] - 'A' + 10;
	 else if((instrg[0] >= 'a')&(instrg[0]<='f')) highnibble = instrg[0] - 'a' + 10;
	 else highnibble = instrg[0] - '0';

	 //  Convert low byte
	 if((instrg[1] >= 'A')&(instrg[1]<='F')) lownibble = instrg[1] - 'A' + 10;
	 else if((instrg[1] >= 'a')&(instrg[1]<='f')) lownibble = instrg[1] - 'a' + 10;
	 else lownibble = instrg[1] - '0';
	 
	 highnibble = highnibble<<4;
	 return highnibble + lownibble;
}

// Read large hex value (32 bits) - skip leading space characters
//   Ignore leading "0x" and terminate on CR, LF, ',' or ' ' characters
unsigned long getlonghex(void)
{
	unsigned char inbyte;
	char instrg[12],inchar;
	unsigned char strg[3];
	int i,cnt,multiplier,skip;
	unsigned long address;
	
	skip = 0;
	// skip leading space characters
	while((inchar=getchar())==' ')  putchar(inchar);
	instrg[0] = inchar;
	instrg[1] = 0x00;
	putchar(inchar);

	//  Read characters until termination character found
	for(i=1; i<11; i++)  {
		inchar = getchar();
		if((inchar==' ')||(inchar==CR)||(inchar==',')||(inchar==LF)) {
			instrg[i] = 0x00;
			break;
			}
		instrg[i] = inchar;
		putchar(inchar);
		}
	putchar(inchar);
		
	//  Strip leading "0x" if present
	if((instrg[0]=='0')&&((instrg[1]=='x')||(instrg[1]=='X')))
	   for(i=2; i<12; i++)  instrg[i-2] = instrg[i];
	// Convert hex starting at end of string

	cnt = 0;
	for(i=0; i<12; i++)  {
		if(instrg[i] == 0) break;
		cnt++;
		}
	// Convert hex pairs (checking for isolated character)
	strg[2] = 0;
	address = 0;
	multiplier = 1;
	while(cnt>0) {
		strg[1] = instrg[cnt-1];
		cnt--;
		if(cnt>=0) { strg[0] = instrg[cnt-1]; cnt--; }
		else {strg[0] = '0'; cnt--; }
		inbyte = hextobyte(strg);
		if(multiplier==1) address += inbyte;
		else if(multiplier==2) address += inbyte*256;
		else if(multiplier==3) address += inbyte*65536;
		else if(multiplier==4) address += inbyte*16777216;
		multiplier++;
		}
	
	return address;
}


int getint(void)
{
	char instrg[10],inchar;
	int i;
	
	// skip leading blanks
	while((inchar=getchar())==' ') {
		putchar(inchar);
		instrg[0] = inchar;
		}
		
	putchar(inchar);
	instrg[0] = inchar;
	
	// collect int values
	for(i=1; i<10; i++)  {
		inchar = getchar();
		if((inchar==' ')||(inchar==CR)||(inchar==',')||(inchar==LF)) {
			instrg[i] = 0x00;
			break;
			}
		instrg[i] = inchar;
		putchar(inchar);
		}
	putchar(inchar);
	return atoi(instrg);
}

char *getstrg(void)
{
	char inchar;
	static char *instrg = "                                        ";
	int i;
	
	i = 0;
	inchar = ' ';
	instrg[0] = 0;
	while((inchar!=CR)&&(inchar!=LF)) {
		inchar = getchar();
		putchar(inchar);
		if((inchar!=CR)&&(inchar!=LF))  instrg[i++] = inchar;
		}
	instrg[i] = 0;
	return instrg;
}

void printf(char *fmt, ...)
{
    va_list ap;
    char *p, *sval,tempval[40];
    int ival;
    unsigned int uival;
    long lval;
    float fval;
    
    va_start(ap,fmt);  // make ap point to 1st unnamed arg
    for(p=fmt; *p; p++) {
    	if(*p!='%') {
    		putchar((unsigned char) *p);
    		continue;
    		}
	    switch(*++p) {
	    	case 'c':
	    	    putchar((unsigned char) *p);
	    	    break;
	    	case 's':
	    	    for(sval=va_arg(ap,char *); *sval; sval++) 
	    	        putchar((unsigned char) *sval);
	    	    break;
	    	case 'd':
	    	    ival = va_arg(ap,int);
     			sprintf(tempval,"%d",ival);
	    	    puts(tempval);
	    	    break;
	    	case 'l':
	    	    lval = va_arg(ap,long);
     			sprintf(tempval,"%d",lval);
	    	    puts(tempval);
	    	    break;
	    	case 'x':
	    	    uival = va_arg(ap,long);
     			sprintf(tempval,"%x",uival);
	    	    puts(tempval);
	    	    break;
	    	case 'f':
	    	    fval = va_arg(ap,float);
     			sprintf(tempval,"%f",fval);
	    	    puts(tempval);
	    	    break;
	    	default:
	    	    putchar((unsigned char) *p);
	    	    break;
	    	}
    	}
	 va_end(ap);  // clean up
}

void scanf(char *fmt, ...)
{
    va_list ap;
    char *p,*ptr,tempstrg[100],inchar;
    float fval;
    
    va_start(ap,fmt);  // make ap point to 1st unnamed arg
    for(p=fmt; *p; p++) {
    	
    	if(*p!='%') continue;
    	
	    switch(*++p) {
	    	//putchar(*p);
	    	case 'c':
	    	    while((inchar=getchar())==' ')  putchar(' ');
	    	    putchar(inchar);
	    	    *va_arg(ap,char *) = inchar;
	    	    break;
	    	case 's':
	    	    //putchar('=');
	    	    strcpy(va_arg(ap,char *),getstrg());
	    	    break;
	    	case 'd':
	    	    *va_arg(ap,int *) = getint();
	    	    break;
	    	case 'l':
	    	    *va_arg(ap,long *) = getint();
	    	    break;
	    	case 'x':
	    	    *va_arg(ap,long *) = getlonghex();
	    	    break;
	    	case 'f':
	    	    sscanf(getstrg(),"%f",&fval);
	    	    *va_arg(ap,float *) = fval;
	    	    break;
	    	default:
	    	    //putchar((unsigned char) *p);
	    	    break;
	    	}
    	}
	 va_end(ap);  // clean up
}

void putchar(unsigned char data)
//int putchar(int data)
{
	unsigned char tmphead;
	
	// Do polled transmit if 1 else interrupt version
	if(0) {
		while(!(*pUART_LSR&0x0040)) ;
		*pUART_THR = data;
		}
	else {
		
	/* Calculate buffer index */
	tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
	/* Wait for free space in buffer */
	while (tmphead == UART_TxTail) ;
	
	UART_TxBuf[tmphead] = data; // store data in buffer 
	UART_TxHead = tmphead;      // store new index
	
//	*pUART_THR = UART_TxBuf[tmphead];
	
	//  Fire TX interrupt
	*pUART_IER |= ETBEI;
//	*pSIC_IMASK = 0x0000C000;
	}
}


//  Read uart - no wait
unsigned char getcharnw(void)
{
	unsigned char tmptail,inchar;
	char rxtmp;
	

	// Do polled receive if 1 else interrupt version
	if(0) {
		if(*pUART_LSR&0x0001) {
			inchar = *pUART_RBR;
			return inchar;
			}
		else return 0; 
		}
	else { 
/*	while ( UART_RxHead == UART_RxTail ) ; */ /* wait for incomming data */
	if(UART_RxHead==UART_RxTail) return 0; /* nothing to get */
		
	tmptail = UART_RxTail + 1; /* calculate buffer index */
	if(tmptail>=UART_RX_BUFFER_SIZE) tmptail = 0; 
	UART_RxTail = tmptail; /* store new index */
	
 
//	if(loopcnt++>100) {
//		putchar('+');
//		loopcnt = 0;
//		}

	asm("cli r0;");
	rxcnt--;
	rxtmp = rxcnt;
	asm("sti r0;");

	if(checkflag == YES)  {
		//putchar('@');	
		checkflag = NO; 
		}
//	if(rxtmp < 0) putchar('<');
//	if(rxtmp > 0x3f) putchar('>');
	
	//  Allow tx at remote serial device again - buffer has more space
	if((rxtmp<(UART_RX_BUFFER_SIZE-50))&&(xoffsent==YES)&&(enablexonoff==YES)) {
		if(xonsent==NO);  {
			putchar(XON); putchar('$');
			xonsent = YES;
			}
//		xoffsent = NO;
		}
	return UART_RxBuf[tmptail]; /* return data */
	}
}


// Getchar waits until a valid character is received
unsigned char getchar(void) 
{
	unsigned char inchar;
	
 	while(!(inchar=getcharnw())) ;
 	return inchar;
}


void puts(char *instrg)
{
	int i;
	
	i = 0;
	while(instrg[i++] != 0x00)  putchar(instrg[i-1]);
}

//  Print integer - charcnt is number of characters to display, leadchartype 
//   is either '0' or ' '.  If leadchartype is 0x00 suppress leading characters
void printint(int intvalue,char charcnt,char leadchartype)
{
 	 char outvalue[6],nolead;
 	 int  i,tenthousands,thousands,hundreds,tens;
	 
	 if(intvalue < 0) {
	 	  putchar('-');
	 	  intvalue *= -1;
		  }
	 tenthousands = intvalue/10000;
	 intvalue -= (tenthousands*10000);
	 thousands = intvalue/1000;
	 intvalue -= (thousands*1000);
	 hundreds = intvalue/100;
	 intvalue -= (hundreds*100);
	 tens = intvalue/10;
	 nolead = 0x00;
	 // TODO - suppress only leading zero's
	 if(tenthousands)  {outvalue[0] = '0' + tenthousands; nolead=0x01;}
	 else outvalue[0] = leadchartype;
	 if(thousands|nolead) {outvalue[1] = '0' + thousands;nolead=0x01;}
	 else outvalue[1] = leadchartype;
	 if(hundreds|nolead) {outvalue[2] = '0' + hundreds;nolead=0x01;}
	 else outvalue[2] = leadchartype;
	 if(tens|nolead) outvalue[3] = '0' + tens;
	 else outvalue[3] = leadchartype;
	 outvalue[4] = '0' + intvalue%10; 
	 outvalue[5] = 0x00;
	 for(i=0; i<charcnt; i++) {
	 	if(outvalue[(5-charcnt)+i]) putchar(outvalue[(5-charcnt)+i]);
	 	}
}

void bytetohex(unsigned char byte)
{
    unsigned char inbyte,inchar;
	
	inbyte = byte & 0x0f;
	if(inbyte > 9) hexbyte[1] = (inbyte - 9) + 0x40;
	else hexbyte[1] = 0x30 + inbyte;
	inbyte = byte & 0xf0;
	inbyte >>= 4;
	if(inbyte >9) hexbyte[0] = (inbyte - 9) + 0x40;
	else hexbyte[0] = 0x30 + inbyte;
    hexbyte[2] = 0x00;
}

void addresstohex(unsigned int address)
{
 	 unsigned char lowbyte,highbyte;
	 int intbyte;
	 
	 lowbyte = address & 0x00FF;
	 bytetohex(lowbyte);
	 hexaddress[2] = hexbyte[0];
	 hexaddress[3] = hexbyte[1];
	 intbyte = address;
	 highbyte = address/256;
	 bytetohex(highbyte);
	 hexaddress[0] = hexbyte[0];
	 hexaddress[1] = hexbyte[1];
	 hexaddress[4] = 0x00;
}

void PrintHexByte(unsigned char inchar)
{
	bytetohex(inchar);
	puts(hexbyte);
}

void PrintHexWord(unsigned int invalue)
{
	addresstohex(invalue);
	puts(hexaddress);
}

// Initialize real time clock
void Init_RealTimeClock(void)
{
	*pRTC_PREN = 0x0001;
	year = 2004;
	
	// TODO - set clock to default value
	*pRTC_STAT = 0x00020000;
}

// Get Date and time from serial port
void Get_DateTime(void)
{
	long datetime;
	
	datetime = 0;
	datetime = *pRTC_STAT;
	second = datetime & 0x0000003F;
	minute = (datetime >> 6) & 0x0000003F;
	hour = (datetime >> 12) & 0x0000001F;
	day = (datetime >> 17) & 0x00007FFF;
}

// Set current date and time 
//  Format is 10/9/04 13:10:30
void Set_DateTime(void)
{
	long newdatetime;
	
	newdatetime = day*131072 + hour*4096 + minute*64 + second;
	*pRTC_STAT = newdatetime;
}

//  Month_day: set month,day from day of year
void month_day(int yr,int yearday, int *pmonth, int *pday)
{
	int i, leap;
	
	leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0;
	for(i=1; yearday > daytab[leap][i]; i++)
		yearday -= daytab[leap][i];
	*pmonth = i;
	*pday = yearday;
}

int day_of_year(int yr, int pmonth, int dayofmonth)
{
	int i,leap;
	
	leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0;
	for(i=1; i < pmonth; i++)
		dayofmonth += daytab[leap][i];
	return dayofmonth;
}

// Return month as ASCII string
char *month_name(int n)
{
	static char *name[]={
		" ",
		"January","February","March",
		"April","May","June",
		"July","August","September",
		"October","November","December" };
	
	return (n<1 || n>12) ? name[0] : name[n];
}

void printdate(void)
{
	int d,m;
	
	month_day(year,day,&m,&d);
	puts(month_name(m)); putchar(' ');
	printint(d,2,0);     puts(", ");
	printint(year,4,0);  putchar(' ');	
}

void printdatetime(void)
{
	printdate();  putchar(' ');
	printint(hour,2,'0'); putchar(':');
	printint(minute,2,'0'); putchar(':');
	printint(second,2,'0');

}

char echochar(void)
{
	char inchar;
	
	inchar = getchar();
	putchar(inchar);
	return inchar;
}

void Enter_DateTime(void)
{
	char inchar,strg[3];
	int month,yearstub,dayofmonth;
	
	strg[2] = 0;
	if((inchar=getchar())!=CR) {
		// get month
		strg[0] = inchar; putchar(inchar);
		strg[1] = echochar();
		month = atoi(strg);
		echochar();  // read '/'
		// get day
		strg[0] = echochar();
		strg[1] = echochar();
		dayofmonth = atoi(strg);
		day = day_of_year(year,month,dayofmonth);
		echochar();  // read '/'
		// get year
		strg[0] = echochar();
		strg[1] = echochar();
		yearstub = atoi(strg);
		if(yearstub>30) year = yearstub + 1900;
		else year = yearstub + 2000;
		echochar();  // read ' '
		// get hour
		strg[0] = echochar();
		strg[1] = echochar();
		hour = atoi(strg);
		echochar();  // read ':'
		// get minute
		strg[0] = echochar();
		strg[1] = echochar();
		minute = atoi(strg);
		echochar();  // read ':'
		// get second
		strg[0] = echochar();
		strg[1] = echochar();
		second = atoi(strg);
		
		// wait until carriage return to exit
		while((inchar=getchar())!=CR) putchar(inchar);
		Set_DateTime();		
		}
}

// Initialize watchdog timer
void InitWatchdog(void)
{
	
}


void WriteBlackfinMemory(long address,unsigned char byte)
{
	*(pSDRAMBot+address) = byte;
}

unsigned char ReadBlackfinMemory(long address)
{
 	unsigned char flashbyte,rbyte;
	
	rbyte = *(pSDRAMBot+address);
	return (rbyte);                              
}


void printlonghex(unsigned long address)
{
	unsigned char inbyte;
	
	puts("0x");
	inbyte = address/16777216;
	bytetohex(inbyte);  puts(hexbyte);
	inbyte = (address/65536)&0x000000FF;
	bytetohex(inbyte);  puts(hexbyte);
	inbyte = (address/256)&0x000000FF;
	bytetohex(inbyte);  puts(hexbyte);
	inbyte = address&0x000000FF;
	bytetohex(inbyte);  puts(hexbyte);
}

void DisplayIntelRecord(unsigned int address,int numbytes,unsigned char *data)
{ 	 
	unsigned char i,byte,ascii[32],asciicnt,crcvalue,tempvalue;
	unsigned int newaddress;

	crcvalue = asciicnt = 0;
	putchar(':');
	
	//  Output bytes on line
	byte = numbytes;
	bytetohex(byte);
	crcvalue += byte;
	puts(hexbyte);
	
	//  Output hex start address
	addresstohex(address);
	tempvalue = (address&0xff00)>>8;
	crcvalue += tempvalue;
	tempvalue = address&0x00ff;
	crcvalue += tempvalue;
	puts(hexaddress);
	
	//  Output 00
	puts("00");
	
	//  Output data
	for(i=0; i<numbytes; i++) {
	    byte = data[i];
	    ascii[asciicnt++] = byte;
		crcvalue += byte;
	    bytetohex(byte);
	 	puts(hexbyte);
	    }
	asciicnt = 0;
	
	// Output CRC
	bytetohex((~crcvalue)+0x01);
	puts(hexbyte);
	if(intelhexprintmode==LONGMODE) {
	   puts("  ");
	   for(i=0; i<numbytes; i++) {
	       if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
	       else putchar('.');
	       }
	   }
	puts("\n\r");  
}


//  Display P1 Blackfin  memory
//
//    If extendedaddress nonzero or numbyte > 65536 then output 
//    Extended Address Record at the appropriate time
//    Output 16 bytes per line
void DisplayBlackfinSDRAMMemory(long startaddress, long numbytes)
{
	 unsigned char firstrecord,databytes[INTELHEXBYTESPERLINE];
	 int i;
	 long recordaddress,endaddress,address,extendedaddress,oldextendedaddress;;

     address = startaddress;
     endaddress = startaddress + numbytes;
     oldextendedaddress = address/65536;

	 bytesonline = INTELHEXBYTESPERLINE;
	 firstrecord = YES;

	 while(address<endaddress) {
	 	 extendedaddress = address/65536;
	     recordaddress = address - extendedaddress*65536; 
		 
		 // Output extended record if start address > 65536
		 if((firstrecord==YES)&&(extendedaddress>0))  {
			 //puts("-1-");
			 firstrecord = NO;
		     DisplayExtendedAddress(extendedaddress);
			 }	
			 
		 // Traversed 65536 boundary?			 
	     else if(extendedaddress>oldextendedaddress) {
			 //puts("-2-");
		     oldextendedaddress = extendedaddress;
			 DisplayExtendedAddress(extendedaddress);
		 	 }
		
		 // Generate current Hex record
		 else {
		     firstrecord = NO;
		     if((recordaddress+bytesonline)>65536) {
				 //puts("-3-");
			     bytesonline = (65536 - recordaddress);
			     
	     		 for(i=0; i<bytesonline; i++) databytes[i] =  *(pSDRAMBot + address + i);
	     		 
			     DisplayIntelRecord(recordaddress,bytesonline,databytes);
			     address += bytesonline;
				 bytesonline = INTELHEXBYTESPERLINE;
				 }
			 else if((recordaddress+(long)bytesonline + extendedaddress*65536)>endaddress) {
				 //puts("-3-3-");
			     bytesonline = endaddress - (recordaddress + extendedaddress*65536);

	     		 for(i=0; i<bytesonline; i++) databytes[i] = *(pSDRAMBot + address + i);
	         
			     DisplayIntelRecord(recordaddress,bytesonline,databytes);
			     address += bytesonline;
				 bytesonline = INTELHEXBYTESPERLINE;
				 }
			 else {  
			 	 //puts("-4-");
			 	 for(i=0; i<bytesonline; i++) databytes[i] = *(pSDRAMBot + address + i);
			     DisplayIntelRecord(recordaddress,bytesonline,databytes); 
			     address += bytesonline; 
				 }
			 }
		}			 
		 
 
	 puts(":00000001FF\n\r");
}

//  Display Blackfin processor SDRAM or Flash memory
void PrintMemory(void)
{
    unsigned char inchar;
    unsigned int address;
    long longaddress,length;
	
	inchar=getchar();
	putchar(inchar);
	switch (inchar) {
		case 's': case 'S':
		    puts("\n\rDisplay Blackfin memory from [address length]: ");
		    longaddress = getlonghex();
		    length = getlonghex();
		    puts("\n\r");
		    DisplayBlackfinSDRAMMemory(longaddress,length);
		    extendedaddress = 0;
		    break;
		case 'f': case 'F':
		    puts("\n\rDisplay Serial Flash memory from [address length]: ");
		    longaddress = getlonghex();
		    length = getlonghex();
		    puts("\n\r");
		    DisplaySerialFlash(longaddress,length,READCONTINUOUS);
//		    PrintFlash(address,length,READBUFFER);
		    extendedaddress = 0;
		    break;
		default:
		    puts("\n\rUnsupported command\n\r");
		    break;
	    }
}

//  Read 2 ASCII Hex characters and return as byte
unsigned char gethexbyte(void)
{
	unsigned char strg[3],inbyte;
	
	strg[0] = getchar(); if(intelhexecho) putchar(strg[0]);
	strg[1] = getchar(); if(intelhexecho) putchar(strg[1]);
	strg[2] = 0;
	inbyte = hextobyte(strg);
	crc += inbyte;
	return inbyte;
}

// Read 2 ASCII Hex CRC characters and return as byte
int checkcrc(void)
{
 	unsigned char crcbyte,strg[3];
	
 	strg[0] = getchar();  if(intelhexecho) putchar(strg[0]); 
	strg[1] = getchar();  if(intelhexecho) putchar(strg[1]);
	strg[2] = 0;
	crcbyte = hextobyte(strg);
 	return crcbyte;
}

//  Read for ASCII hex characters and convert to int address
unsigned int getinteladdress(void)
{
	unsigned char strg[3],addressbyte;
	unsigned int address;
	
	addressbyte = gethexbyte();
	address = gethexbyte();
	address += addressbyte*256;
	return address;
}

// Convert Intel Hex ASCII string to binary
// :10 0100 00 0C948F0D0C941D0E0C941D0E0C941D0E 38
// :1001000000C941D0E0C941D0E0C944E590C941D0E38
//  This routine decodes a line of Intel Hex - support
//  for extended address and end of file records is included
int DecodeIntelHex(void)
{
 	 unsigned char strg[8];
 	 unsigned char addressbyte,inbyte;
	 int i;
	 
	 // Reset CRC
	 crc = 0;
	 endofintelhex = NO;
	 
	 // Convert count of bytes on line
	 bytesonline = gethexbyte();
	 
	 // Convert address to binary - ignore filler byte
	 // Typical Intel Hex consists of 4 ASCII characters
	 inteladdress = getinteladdress();
	 
	 // Read record type (2 characters)
	 recordtype = gethexbyte();
	 
	 // Check the record type and process accordingly
	 // If record type is extended address (bits 31-16) convert to binary
	 if(recordtype == 4)  extendedaddress = getinteladdress();
	 
	 // If end of file record reset extended address to zero
	 else if(recordtype == 1)  {
	 	extendedaddress = 0;
	 	endofintelhex = YES;
//	 	puts("End of Hex record found\n\r");
	 	}
	 
	 // else must be standard data record
	 else {
	 	for(i=0; i<bytesonline; i++) intelbinary[i] = gethexbyte();
	 	}
		 
	// Check CRC - todo
	if(checkcrc() == crc) return 1;
	else return -1;
}

// Load Intel Hex file into SDRAM
void ProgramSDRAM(void)
{
     int i;
	 
	 for(i=0; i<bytesonline; i++)  WriteBlackfinMemory(inteladdress+i,intelbinary[i]);
}


//  Called by ':' command from serial port 
//
//  Programming data fed line by line.  For SDRAM program each line
//  as it is received - for Flash accumulate enough data to fill 
//  a buffer then program buffer.  Note XON/XOFF flow control should 
//  be enabled when programming flash due to the 20msec burn time
//  per 264 byte buffer.
void ProgramMemory(void)
{
	if(endofintelhex==YES)  puts("\n\r");
	if(intelhexecho==YES)  putchar(':');
	endofintelhex = NO;
 	DecodeIntelHex();
	switch (programmode) {
		case LBlackfinSDRAM:
		    ProgramSDRAM();
			break;
		case LFLASH:
		    ProgramFlash();
			break;
		} 
}

//  Read ASCII hex value and convert to byte
unsigned char gethextobyte(void)
{
	unsigned char hexbyte;
	unsigned char instrg[10],inchar;
	int i;
	
	for(i=0; i<5; i++)  {
		inchar = getchar();
		if((inchar == CR)||(inchar==LF)) {
			instrg[i] = 0x00;
			break;
			}
		instrg[i] = inchar;
		putchar(inchar);
		}
	puts("\n\r");
	for(i=0; i<2; i++) instrg[i] = instrg[i+2];
	instrg[2] = 0;
	return hextobyte(instrg);
}

// Fill SDRAM or SRAM with a byte value - enter base address and length
void FillSDRAMMemory(long address,long length,unsigned char databyte)
{
	long i;
	
	i = 0;
	while(length-->=0) *(pSDRAMBot+address+i++) = databyte;
}


//  Fill Blackfin SDRAM with a byte from start address of length
//
//  TODO - fill Flash
void FillMemoryCommand(void)
{
	unsigned char databyte,inchar;
	unsigned long longaddress,length;
	
	// Fill Flash or SDRAM?
	inchar = getchar();
	putchar(inchar);
	puts("\n\rFill Memory: enter address, length, byte (hex): "); 
	// Read address and convert to int address and extended address
	longaddress = getlonghex();
	length = getlonghex();
	databyte = getlonghex();
	if(inchar=='s'||inchar=='S') FillSDRAMMemory(longaddress,length,databyte);
	else if(inchar=='f'||inchar=='F') FillFlashMemory(longaddress,length,databyte,FROMCONST);
	puts("\n\r");
}


void LoadMemory(void)
{
    unsigned char inchar;
	
	inchar = getchar();
	putchar(inchar);
	switch (inchar) {
		case 'f': case 'F':
//			putchar(inchar);
		    puts("\n\rSet Load Flash as default\n\r");
			programmode = LFLASH;
		    break;
		case 's': case 'S':
//			putchar(inchar);
		    puts("\n\rSet Load SDRAM as default\n\r");
			programmode = LBlackfinSDRAM;
		    break;
		default:
		   break;
	    }
}

//  Main memory to memory move command
//
//    Moves to/from Flash require special procedures
void MoveMemory(char source, char destination,long saddress,long length,long daddress)
{
	int i;
	
	if(source=='S') source = 's';
	if(destination=='F') destination = 'f';
	
	//  SDRAM to SDRAM - direct move
	if(source=='s'&&destination=='s') {
		puts("Transfer from SDRAM to SDRAM\n\r");
		for(i=0; i<length; i++)  *(pSDRAMBot+i+daddress) = *(pSDRAMBot+i+saddress);
		}
	//  SDRAM to Flash - Flash writes must be handled in blocks
	else if(source=='s'&&destination=='f') {
		puts("Transfer from SDRAM to Flash\n\r");
		fillindex = saddress;
		FillFlashMemory(daddress,length,0x00,FROMSDRAM);
		}
	//  Flash to SDRAM - TODO add Flash chip traverse
	else if(source=='f'&&destination=='s') {
		FlashChipSelect(1,LOW);
		SPI_StartReadFlashPage(saddress);  // continuous read command 0x68
		while(length--) *(pSDRAMBot+daddress++) = SPI_MasterReceive();
//		puts("\n\rlast address=");	printlonghex(pSDRAMBot+daddress-1);
//		puts("\n\r");
		printf("\n\rlast address=%x\n\r",pSDRAMBot+daddress-1);
		FlashChipSelect(1,HIGH);
		}
	//  Flash to Flash - Flash writes must be handled in blocks
	else if(source=='f'&&destination=='f') {
		puts("Function not yet implemented\n\r");
		}
}

//  Memory to memory move command parser
void MoveMemoryCommand(void)
{
	char source,destination;
	long sourceaddress,destinationaddress,length;
	
	puts("\n\rEnter source and destination memory [S|F-S|F]: ");
	source = getchar();
	putchar(source);
	destination = getchar();
	putchar(destination);
	puts("\n\rEnter source address, length and destination address: ");
	sourceaddress = getlonghex();
	length = getlonghex();
	destinationaddress = getlonghex();
	puts("\n\r");
	MoveMemory(source,destination,sourceaddress,length,destinationaddress);	
}

// Output program version number
void ProgramVersion(void)
{
    char majorversion[3],minorversion[4];
    unsigned char inbyte;
	  
	puts("\n\rP1 Monitor Version ");
	printint(MAJORVER,1,'0'); putchar('.');
	printint(MINORVER,2,0); puts(", ");
	
	// Get flash type
	inbyte = FlashStatus(1);
	if((inbyte&FLASHIDMASK)==AT45DB041ID) puts("F1-AT45DB041");
	else if((inbyte&FLASHIDMASK)==AT45DB081ID) puts("F1-AT45DB081");
	else if((inbyte&FLASHIDMASK)==AT45DB161ID) puts("F1-AT45DB161");
	inbyte = FlashStatus(2);
	if((inbyte&FLASHIDMASK)==AT45DB041ID) puts(", F2-AT45DB041");
	else if((inbyte&FLASHIDMASK)==AT45DB081ID) puts(", F2-AT45DB081");
	else if((inbyte&FLASHIDMASK)==AT45DB161ID) puts(", F2-AT45DB161");
	else puts(", F2-No Flash"); 
	
	// Get memory size
	puts("\n\r");
}


//  Using 118.8MHz SCLK counter clocks 119 counts per microsecond
//  Maximum count period is 32 seconds
void CountPeriodStart(void)
{
	unsigned long countperiod;
	
	countperiod = (long) 32000000;
	countperiod *= 131;
	//  Use timer 2 as count up timer
	*pTIMER2_CONFIG	= 0x0049;
	*pTIMER2_PERIOD	= countperiod;
	*pTIMER2_WIDTH = 0x00000006;
	*pTIMER_ENABLE |= 0x0004;
	// Timer stops once period count is reached
	*pTIMER_DISABLE |= 0x0004;
}

//  Return count period in microseconds and reset counter
long CountPeriod(void)
{
	long countperiod;
	
	countperiod = *pTIMER2_COUNTER/131;
	*pTIMER_DISABLE &= 0x000B;	
	*pTIMER_ENABLE &= 0x000B;
	*pTIMER_STATUS |= 0x4000;
	return countperiod;
}

//  Return count period in microseconds and reset counter
long CountUPeriod(void)
{
	long countperiod;
	
	countperiod = *pTIMER2_COUNTER;
	*pTIMER_DISABLE &= 0x000B;	
	*pTIMER_ENABLE &= 0x000B;
	*pTIMER_STATUS |= 0x4000;
	return countperiod;
}


//  Exercise most of the Blackfin functions
void TestBlackfin(void)
{
	unsigned int temp,temp2;
	int failcnt;
	long i,badmatchcnt,bankcnt,bankbadcnt;
	
	puts("\n\rTest P1 Blackfin\n\r");
	puts(" Test RAM");
		puts(" - Passed\n\r");
		
	//  Test SDRAM by writting 0x55 and 0xAA to all locations
	//     Non-destructive test
	puts(" Test SDRAM");
	badmatchcnt = 0;
	bankcnt = 0;
	// Byte access
	puts("\n\r  Byte Access test");
	while(bankcnt<512) {
		bankbadcnt = 0;
	    for(i=bankcnt*65536; i<bankcnt*65536+65536; i++) {
		    temp = *(pSDRAMBot+i);
		    *(pSDRAMBot+i) = 0x55;
		    if(*(pSDRAMBot+i)!=0x55) bankbadcnt++;
		    *(pSDRAMBot+i) = 0xAA;
		    if(*(pSDRAMBot+i)!=0xAA) bankbadcnt++;
		    *(pSDRAMBot+i) = temp;
		    }
		bankcnt++;
		badmatchcnt += bankbadcnt;		
	    if(bankbadcnt) {
	    	puts("\n\r  Bank ");  printint(bankcnt,4,0); putchar(' ');
			puts(" failed with ");
			printint(bankbadcnt,5,0);
			puts(" mismatches");
			}
		bankbadcnt = 0;
		}
	//  Output results
	if(badmatchcnt) {
		puts("\n\r  SDRAM failed with ");
		printint(badmatchcnt,6,0);
		puts(" mismatches\n\r");
		}
	else puts(" - Passed\n\r");
	// Word access
	puts("  Word Access test");
	badmatchcnt = 0;
	bankcnt = 0;
	while(bankcnt<256) {
		failcnt = 0;
		bankbadcnt = 0;
	    for(i=bankcnt*65536; i<bankcnt*65536+65536; i++) {
		    temp = *(pSDRAMWordBot+i);
		    *(pSDRAMWordBot+i) = 0x5555;
		    temp2 = *(pSDRAMWordBot+i);
		    if(temp2 != 0x5555) {
		    	bankbadcnt++;
		    	failcnt++;
		    	if((failcnt<1100)&&(bankcnt==255)) {
		    		puts("\n\r  Failed 5 at ");
		    		printint(i%65536,5,0);
		    		puts(" wrote 0x5555 - read ");
		    		*(pSDRAMWordBot+i) = 0x5555;
		    		temp2 = *(pSDRAMWordBot+i);
		    		addresstohex(temp2);
		    		puts(hexaddress);
		    		}
		    	}
		    *(pSDRAMWordBot+i) = 0xAAAA;
		    if(*(pSDRAMWordBot+i)!=0xAAAA) {
		    	bankbadcnt++;
		    	failcnt++;
		    	if((failcnt<1100)&&(bankcnt==255)) {
		    		puts("\n\r  Failed A at ");
		    		printint(i%65536,5,0);
		    		}
		    	}
		    *(pSDRAMWordBot+i) = temp;
		    }
		bankcnt++;
		badmatchcnt += bankbadcnt;		
	    if(bankbadcnt) {
	    	puts("\n\r  Bank ");  printint(bankcnt,4,0); putchar(' ');
			puts(" failed with ");
			printint(bankbadcnt,5,0);
			puts(" mismatches");
			}
		bankbadcnt = 0;
		}
	//  Output results
	if(badmatchcnt) {
		puts("\n\r  SDRAM failed with ");
		printint(badmatchcnt,5,0);
		puts(" mismatches\n\r");
		}
	else puts(" - Passed\n\r");

	
	//  Test Flash 
	puts(" Test Flash");
		puts(" - Passed\n\r");
	puts(" Test UART");
		puts(" - Passed\n\r");
	puts(" Test SPORTs");
		puts(" - Passed\n\r");
	puts(" Test SPI");
		puts(" - Passed\n\r");
	puts(" Test CM64");
		puts(" - Passed\n\r");
}

void BootApplication(void)
{
	unsigned long bootaddress;
	
	puts("\n\rAddress of Application to Boot [32bit hex]: ");
	bootaddress = getlonghex();
	
	//  Pass to .ldr loader program
	//    The boot image must conform to the Analog Device Blackfin .ldr format
	//    This process may overwrite the boot monitor
	//    Registers are loaded onto the stack - if the new application executes 
	//    an exit or return it should come back to the BootMonitor but this 
	//    is not guaranteed.
	LoadElfImage(bootaddress);
	
}

void ExtendedMessage(void)
{
	puts("\n\rP1 Pirana Extended Commands\n\r\n\r"); 
	puts("   i  - 2000 init timing\n\r");
	puts("   g  - Demo of I2C init\n\r");
	puts("   0  - Flash Status\n\r");
	puts("   1  - Write byte to CM64\n\r");
	puts("   2  - Init CM64\n\r");
	puts("   3  - Halt video DMA/PPI operation\n\r");
	puts("   4  - YUV to RGB timing test\n\r");
	puts("   5  - Read SAA7113 ID continuously\n\r");
	puts("   6  - Read SAA7113 status continuously\n\r");
	puts("   7  - Reset SAA7113 continuously\n\r");
	puts("   8  - SPI transmit test\n\r");
	puts("   9  - SAA7113 Status\n\r");
	puts("   H  - Help Menu\n\r");
}


// Output default P1 Blackfin Sign-on message
void SignOnMessage(void)
{
	printf("\n\rP1 Blackfin Monitor v%d.%d\n\r",MAJORVER,MINORVER);
	puts("Copyright 2004-2007, SOC Robotics, Inc.\n\r\n\r");
	puts("   D[S|F]  - Display [S]DRAM or [F]lash memory\n\r");
	puts("   L[S|F]  - Set default memory type for Intel Hex Load to [S]DRAM or [F]lash\n\r");
	puts("   F[S|F]  - Fill [S]DRAM or [F]lash memory [address,length,byte]\n\r");
	puts("   M[S/F|S/F]  - Move [S]DRAM/[F]lash memory to [S]DRAM/[F]lash\n\r");
	puts("                 [source address,length,destination address]\n\r");
	puts("   K   - Compare memory (range)\n\r");
	puts("   T   - Test Blackfin\n\r");
	puts("   B   - Boot Application (address,.ldr format)\n\r");
	puts("   P   - Display/Set Preferences\n\r");
	puts("   C   - Display/Set Blackfin Clock\n\r");
	puts("   X   - Display/Set new Internet Settings\n\r");
	puts("   V   - Version\n\r");
	puts("   S   - P1 Specifications\n\r");
	puts("   E   - Extended commands\n\r");
	puts("   H   - Help Menu\n\r");
}

//  Convert 4 byte binary IP value to ASCII format
void DecodeIP(void)
{
	int i;
	
	for(i=0; i<3; i++)  {
		printint(ip[i],3,0);
		putchar('.');
		}
	printint(ip[3],3,0);
}

//  Convert IP ASCII string to 4 byte binary value
void EncodeIP(char *instrg)
{
	char strg[4];
	int i,ptr,cnt;
	
	cnt = 0;
	ptr = 0;

	while(cnt<3) {
		i = 0;
	    while((strg[i]=instrg[ptr])!='.')  {
		    i++;
		    ptr++;
	    	}
	    strg[i] = 0;
	    i++;
	    ptr++;
	    ip[cnt] = atoi(strg);
	    cnt++;
		}
	i = 0;
	while((strg[i]=instrg[ptr])!=0)  {
		i++;
		ptr++;
		}
	strg[i] = 0;
	ip[3] = atoi(strg);	
}

//  Convert Ethernet MAC ASCII address to binary string
void EncodeMAC(char *instrg)
{
	unsigned char strg[3];
	int i,ptr;
	
	ptr = 0;
	strg[2] = 0;
	for(i=0; i<12; i+=2) {
		strg[0] = instrg[i];
		strg[1] = instrg[i+1];
		ethernetmac[ptr++] = hextobyte(strg);
		}
}

//  Convert Ethernet MAC address binary string the ASCII string
void DecodeMAC(void)
{
	int i;
	
	for(i=0; i<6; i++)  {
		bytetohex(ethernetmac[i]);
		puts(hexbyte);
		}
}

//  Check for Ethernet settings stored in AVR else use
//  startup defaults
void Init_Internet(void)
{
	int i;
	
	EncodeMAC(ETHERNETMACID); 
	EncodeIP(IPADDRESS);  for(i=0; i<4; i++)  ipaddress[i] = ip[i];
	EncodeIP(IPMASK);     for(i=0; i<4; i++)  ipmask[i] = ip[i];
	EncodeIP(IPGATEWAY);  for(i=0; i<4; i++)  ipgateway[i] = ip[i];
}


void InternetSettings(void)
{
	int i;
	
	Init_Internet();
	puts("\n\rInternet Settings\n\r");
	puts("  Ethernet MAC: "); DecodeMAC(); puts("\n\r");
	puts("  IP Address:   "); 
		for(i=0; i<4; i++)  ip[i] = ipaddress[i];
		DecodeIP(); puts("\n\r");
	puts("  IP Mask:      "); 
	    for(i=0; i<4; i++)  ip[i] = ipmask[i];
	    DecodeIP(); puts("\n\r");
	puts("  IP Gateway:   "); 
	    for(i=0; i<4; i++)  ip[i] = ipgateway[i];
	    DecodeIP(); puts("\n\r");
	puts("  User ID:      "); puts(USERID); puts("\n\r");
	puts("  Password:     "); puts(PASSWORD); puts("\n\r\n\r");
}

void GetInternetSettings(void)
{
	char *instrg;
	int i;
	
	//  Display/change Ethernet MAC Address
	puts("\n\rEthernet MAC ("); DecodeMAC(); puts("): ");
	instrg = getstrg();
	puts("\n\r");	
	if(instrg[0])  EncodeMAC(instrg); 
		
	//  Display/change IP address
	puts("IP address ("); 
	for(i=0; i<4; i++)  ip[i] = ipaddress[i];
	DecodeIP(); puts("): ");
	instrg = getstrg();
	puts("\n\r");
	if(instrg[0])  {
		EncodeIP(instrg);  
		for(i=0; i<4; i++)  ipaddress[i] = ip[i];
		}
	
	//  Display/change IP mask
	puts("Mask ("); 
	for(i=0; i<4; i++)  ip[i] = ipmask[i];
	DecodeIP(); puts("): ");
	instrg = getstrg();
	puts("\n\r");	
	if(instrg[0])  {
		EncodeIP(instrg);     
		for(i=0; i<4; i++)  ipmask[i] = ip[i];
		}
	
	//  Display/change IP gateway
	puts("Gateway ("); 
	for(i=0; i<4; i++)  ip[i] = ipgateway[i];
	DecodeIP(); puts("): ");
	instrg = getstrg();
	puts("\n\r");	
	if(instrg[0])  {
		EncodeIP(instrg);  
		for(i=0; i<4; i++)  ipgateway[i] = ip[i];
		}
}


//  Compare Memory location range
void CompareMemory()
{
	long sourceaddress,destinationaddress,length,i,mismatchcount;
	
	mismatchcount = 0;
	puts("\n\rEnter source address, length and destination address: ");
	sourceaddress = getlonghex();
	length = getlonghex();
	destinationaddress = getlonghex();
	puts("\n\r");
	for(i=0; i<length; i++) {
		if(*(pSDRAMBot+i+sourceaddress) != *(pSDRAMBot+i+destinationaddress))  {
			puts("Mismatch at address: ");
			PrintHexWord(i+sourceaddress);
			puts("\n\r");
			mismatchcount++;
			}
		}
	if(mismatchcount) {
		printint(mismatchcount,6,' '); 
		puts(" mismatches");
		puts("\n\r");
		}
	else puts("No mismatches\n\r");	
}

//  Display/set user preferences
//  For the moment do an SDRAM memory compare
void Preferences(void)
{
	char inchar;
	long sourceaddress,destinationaddress,length,i,mismatchcount;

	puts("\n\rPreference Commands\n\r\n\r"); 
	puts("   I[S|L]  - Intel Hex [S]hort/[L]long format\n\r");
	puts("   H  - Help Menu\n\rp-");
	
	inchar = getchar();
	putchar(inchar);
	switch (inchar) {
		case 'i': case 'I':
			inchar = getchar();
			putchar(inchar);
			if((inchar=='s')||(inchar=='S')) {
				intelhexprintmode = SHORTMODE;
				puts("  Intel Hex short mode\n\r");
				}
			else if((inchar=='l')||(inchar=='L')) {
				intelhexprintmode = LONGMODE;
				puts("  Intel Hex long mode\n\r");
				}
			break;
		case 'h': case 'H':
			SignOnMessage(); 
			putchar('>');
			break;
		}	
}

// Non destructive RAM size check
//  P1 available in 4,8,16 or 32Mx16
int SDRAMSize(void)
{
	char sdramexists;
	unsigned char tempbyte;
	int sdramsize = 4;
	
	sdramexists = YES;
	// Check for 4Mx16
	tempbyte = *(pSDRAMBot+0x7FFFFF);
	*(pSDRAMBot+0x7FFFFF) = 0x55;
	if(*(pSDRAMBot+0x7FFFFF)!=0x55) sdramexists = NO;
	*(pSDRAMBot+0x7FFFFF) = 0xAA;
	if(*(pSDRAMBot+0x7FFFFF)!=0xAA) sdramexists = NO;
	if(sdramexists==NO)  return 0;  // no SDRAM
	*(pSDRAMBot+0x7FFFFF) = tempbyte;
	// Check for 8Mx16
	tempbyte = *(pSDRAMBot+0xFFFFFF);
	*(pSDRAMBot+0xFFFFFF) = 0x55;
	if(*(pSDRAMBot+0xFFFFFF)!=0x55) sdramexists = NO;
	*(pSDRAMBot+0xFFFFFF) = 0xAA;
	if(*(pSDRAMBot+0xFFFFFF)!=0xAA) sdramexists = NO;
	if(sdramexists==NO)  return 4;  // 4Mx16 SDRAM
	*(pSDRAMBot+0xFFFFFF) = tempbyte;
	// Check for 16Mx16
	tempbyte = *(pSDRAMBot+0x1FFFFFF);
	*(pSDRAMBot+0x1FFFFFE) = 0x55;
	*(pSDRAMBot+0x1FFFFFF) = 0xAA;
	if(*(pSDRAMBot+0x1FFFFFE)!=0x55) sdramexists = NO;
	if(*(pSDRAMBot+0x1FFFFFF)!=0xAA) sdramexists = NO;
	if(sdramexists==NO)  return 8;  // 8Mx16 SDRAM
	*(pSDRAMBot+0x1FFFFFF) = tempbyte;
	// Check for 32Mx16
	tempbyte = *(pSDRAMBot+0x3FFFFFF);
	*(pSDRAMBot+0x3FFFFFF) = 0x55;
	if(*(pSDRAMBot+0x3FFFFFF)!=0x55) sdramexists = NO;
	*(pSDRAMBot+0x3FFFFFF) = 0xAA;
	if(*(pSDRAMBot+0x3FFFFFF)!=0xAA) sdramexists = NO;
	if(sdramexists==NO)  return 16;  // 16Mx16 SDRAM
	*(pSDRAMBot+0x3FFFFFF) = tempbyte;
	return 32;  // 32Mx16		
}

// Output characteristics of this P1
void Specifications(void)
{	
	puts("\n\rP1 Specifications:\n\r");
	puts("  DSP Processor:  BF532 Blackfin, 394MHz, ");
	puts("Rev 0."); printint((*pDSPID&0x0000000F),2,0); puts("\n\r");
	puts("  SDRAM Memory:   "); printint(SDRAMSize(),2,0);
	puts("Mx16 130MHz\n\r");
	puts("  Flash:          ");
	if(f1pagesize>0&&f2pagesize>0) puts("2 Flash chips, ");
	else puts("1 Flash chip ");
	if(f1bytesize==540672) puts("512Kx8 ");
	else if(f1bytesize==540672*2) puts("1Mx8 ");
	else if(f1bytesize==540672*4) puts("2Mx8 ");
	if(f2bytesize==540672) puts("512Kx8 ");
	else if(f2bytesize==540672*2) puts("1Mx8 ");
	else if(f2bytesize==540672*4) puts("2Mx8 ");
	puts("\n\r");
	puts("  AVR Processor:  8MHz, 16K Flash\n\r\n\r");
	
}

//  Init AVR - TBD
//  The AVR defaults to Slave SPI - the Blackfin is Master
//  AVR gets Blackfin's attention by pulsing PD2 (connected to PF3) low.  Blackfin
//  then requests status from AVR - if AVR has data or needs Blackfins attention 
//  data can be transferred on SPI with Blackfin as Master.  If status indicates that
//  AVR needs control of Serial Flash then Blackfin can honour the request and release 
//  control of SPI by pulling PF3 low after a delay or can ignore the request.  PF3/PD2
//  is pulled high with a 10K resistor so if the AVR releases the attention line and 
//  checks for Blackfin acknowledgement the AVR should not switch SPI from Slave to 
//  Master unless the Blackfin holds PF3 low.  The Blackfin acknowledges that SPI has
//  been released by allowing PF3 to go high.  Note that PF3/PD2 is connected to serial 
//  flash chip 2 chip select so no activity should take place on SPI until the hand shake
//  completes.
int  Init_AVR(void)
{
	// Return 1 if AVR OK and setup or -1 if not
	return 1;
}

//  Setup Blackfin and initialize all peripherals
void Init_Blackfin(void)
{
	//Initialize System Configuration Register
//	sysreg_write(reg_SYSCFG, 0x32);	
	
	//  Initialize periperals and system components	
	//  Note:  For Flashed versions do not reinitialize PLL, SDRAM and Timers
	Init_PLL(); 
	Init_SDRAM(); 
	Init_Timers();
	//  Start code again
	Init_EBIU();       //  Activate APS12 Ethernet Adapter CS
//	Init_PPI_CM64();   //  Make sure Video Port PPI and PF8-15 are inputs
//	Init_UART(57600);
	Init_UART(115200);
//	Init_UART(9600);
		
	Init_Interrupts();
	Init_RealTimeClock();
	Init_MasterSPI();  // Default setting - if the AVR needs to access Serial Flash
					   // then the Blackfin must relinguish control

	// Watchdog timer setup
	InitWatchdog();
	
    // Setup constants and variables
	programmode = LBlackfinSDRAM;
	
	//  Output startup message on UART port
	SignOnMessage();
 	
 	//  Output Ethernet and related configuration information
//	InternetSettings();
	
	//  Set Intel Hex extended address variable to zero and 
	//  initialize Flash programming parameters - SPI Master
	//  Initialize Intel hex download parameters for Flash loading
	ResetFlashVars();
	intelhexecho = 0x01;
	intelhexprintmode = SHORTMODE;
	Init_Flash();
	
	// Check Flash Type(s) and set size parameters
	//  512K,1M or 2M per Flash - sets the various 
	//  parameters that interact with the Flash
	CheckFlash();
	
	//  Initialize the communication protocol to allow communication
	//  with the AVR
	if(Init_AVR()==-1)  puts("AVR not communicating - TBD\n");
	
	//  Display command prompt
	//    Note:  Blackfin command prompt is '>' while AVR prompt is ':'
	putchar('>');
	
}

void GCodeTest(void)
{
    long i,period,loops;
    float x,y,r,angle;
       
	printf("\n\rGcode test - enter iterations: ");
	
	loops = getint();	
  	CountPeriodStart();
  	angle = 0.235;
  	
    for(i=0; i<loops; i++) {
    	x = 10.0;
    	y = 20.0;
    	r = x + y;
    	angle += 0.0001;
    	x = r*sin(angle);
    	y = r*cos(angle);
    	}	
	printf("\n\rElapsed time = %f msec\n\r>",CountPeriod()/1000.0);
}

void SpartanTest(void)
{
	unsigned char inchar,inbyte,outbyte;
	unsigned short outword,inword;
	unsigned int inint,iptr,numptr;
	long mismatchcnt;
	
	printf("\r\nSpartan Test Harness for P2 Design\n\r");
	printf("Commands:\n\r");
	printf("   1  - single read/write to AMS3\n\r");
	printf("   2  - single read/write to AMS2\n\r");
	printf("   3  - write 2048 bytes to AMS2\n\r");
	printf("   4  - write 1024 words to AMS2\n\r");
	printf("   5  - Quick read/write 1024 words to AMS2\n\r");
	printf("   6  - Exhaustive read/write 1024 words to AMS2\n\r");
	printf("   e  - exit Spartan Test Harness\n\r-");
	
	inchar = ' ';
	while(inchar!='e')  {
		inchar = getchar();
		switch(inchar) {
			case '1':
			    printf("\n\rRead/Write to AMS3\n\r-");
				*pAMS3 = 0xaa55;
				*pAMS3B = 0x55;
				//*pAMS2 = 0xaa;
				inint =  *pAMS3;
				inchar =  *pAMS3B;
			    break;
			case '2':
			    printf("\n\rRead/Write to AMS3\n\r-");
				*pAMS2 = 0xaa55;
				*pAMS2B = 0x55;
				//*pAMS2 = 0xaa;
				inint =  *pAMS2;
				inchar =  *pAMS2B;
			    break;
			case '3':
			    printf("\n\rWrite 2048 bytes to AMS2\n\r-");
			    for(iptr=0; iptr<2048; iptr++)  {
			    	*(pAMS2B + iptr) = 0x55;
			    	//*(pAMS2B + iptr) = 0xaa;
			    	}
			    break;
			case '4':
			    printf("\n\rWrite 1024 bytes to AMS2\n\r");
			    for(iptr=0; iptr<1024; iptr++)  {
			    	*(pAMS2 + iptr) = 0x5555;
			    	*(pAMS2 + iptr) = 0xaaaa;
			    	}
			    break;
			case '5':
				printf("   5  - Quick read/write 1024 words to AMS2\n\r");
			    mismatchcnt = 0;
			    for(iptr=0; iptr<1024; iptr++)  {
			    	*(pAMS2 + iptr) = 0x5555;
			    	inword = *(pAMS2 + iptr);
			    	if(inword!=0x5555) mismatchcnt++;
			    	*(pAMS2 + iptr) = 0xaaaa;
			    	inword = *(pAMS2 + iptr);
			    	if(inword!=0xaaaa) mismatchcnt++;
			    	}
			    printf("Number of mismatches = %d\n\r",mismatchcnt);
			    break;
			case '6':
				printf("   6  - Exhaustive read/write 1024 words to AMS2\n\r");
			    mismatchcnt = 0;
			    for(iptr=0; iptr<1024; iptr++)  {
			    	for(numptr=0; numptr<65536; numptr++) {
			    		*(pAMS2 + iptr) = numptr;
			    		inword = *(pAMS2 + iptr);
			    		if(inword!=numptr) mismatchcnt++;
			    		}
			    	}
			    printf("Number of mismatches = %d\n\r",mismatchcnt);
			    break;
			}
		}
}


