// **************************************************************************
//
//  File:  P1_Init.c				Version: 0.51
//
//  Date:  October 20, 2004		
//
//  Initialization code for P1 Blackfin Monitor
//
//  Copyright 2004, SOC Machines, Inc.
//
// **************************************************************************
//#include <cdefBF532.h>
#include <ccblkfn.h>
#include <sysreg.h>
#include <sys\exception.h>		//Interrupt Handling Header
#include "P1_Main.h"
#include "P1_I2C.h"

//  Declarations
#define RAM_Length 189000
#define CLKIN 	27

// UART variables
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars, sendxonxoff;
char  rxcnt, checkflag;


//--------------------------------------------------------------------------//
// Function:	Init_Flags													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures PF8 as input for edge sensitive	//
//				interrupt generation.										//
//				The switch connected to PF8 (SW7) can be used to change the	//
//				direction of the moving light.								//
//--------------------------------------------------------------------------//
void Init_Flags(void)
{
	*pFIO_INEN		= 0x0100;
	*pFIO_DIR		= 0x0000;
	*pFIO_EDGE		= 0x0100;
	*pFIO_MASKA_D	= 0x0100;
}

//--------------------------------------------------------------------------//
// Function:	Init_Timers													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function initialises Timer0 for PWM mode.				//
//				It is used as reference for the 'shift-clock'.				//
//--------------------------------------------------------------------------//
void Init_Timers(void)
{
	*pTIMER0_CONFIG		= 0x0019;
	*pTIMER0_PERIOD		= 0x00800000;
	*pTIMER0_WIDTH		= 0x00400000;
	*pTIMER_ENABLE		= 0x0001;
}

//--------------------------------------------------------------------------//
// Function:	Init_EBIU													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function initialises and enables the asynchronous		//
//				memory banks for the External Bus Interface Unit (EBIU), so	//
//				that access to LAN91C111 10/100 Ethernet Chip is possible	//
//              (AMS3) and AMS0,1,2 and CLKOUT is enabled					//
//--------------------------------------------------------------------------//
void Init_EBIU(void)
{
	*pEBIU_AMBCTL0	= 0x7bb07bb0;
	*pEBIU_AMBCTL1	= 0x7bb07bb0;
	*pEBIU_AMGCTL	= 0x000f;
}

//--------------------------------------------------------------------------//
// Function:	Init_Interrupts												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function initialises the interrupts for Timer0,		//
//				UART and DMA 0 interrupts									//
//--------------------------------------------------------------------------//
void Init_Interrupts(void)
{
	// Assign core IDs to interrupts
	*pSIC_IAR0 = 0xffffffff;
	*pSIC_IAR1 = 0x32ffffff;	// UART Rx -> IVG9, UART Tx -> IVG10
//	*pSIC_IAR1 = 0x324ffff1;	// UART Rx -> IVG9, UART Tx -> IVG10, SPI->IVG11
   								// DMA0 -> IVG8
	*pSIC_IAR2 = 0xfffffff5;	// Timer0 -> ID4; 

	// assign ISRs to interrupt vectors
	register_handler(ik_ivg12, Timer0_ISR);		// Timer0 ISR -> IVG 12	
	register_handler(ik_ivg8, DMA0_PPI_ISR);	// assign DMA0 PPI ISR to interrupt vector 8
	register_handler(ik_ivg9, UART_RX_ISR);		// UART Rx ISR -> IVG 9	
	register_handler(ik_ivg10, UART_TX_ISR);	// UART Tx ISR -> IVG 10	
//	register_handler(ik_ivg11, SPI_ISR);		// UART Tx ISR -> IVG 10	

	// Enable UART Tx (10), Rx (9) interrupts 	
	*pSIC_IMASK = 0x0000C000; // all interrupts 0=disabled
//	*pSIC_IMASK=0x0000C100; // all interrupts 0=disabled but DMA0 PPI interrupt enabled	
}

//--------------------------------------------------------------------------//
// Function:	Init_DMA_CH0												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures the PPI DMA channel to support		//
//				transfer of video input from the CM64.						//
//				The CM64 generates a stream of ITU656 bytes					//
//--------------------------------------------------------------------------//
void Init_DMA_CH0(unsigned char skipmode)
{
	// Target address of the DMA
	*pDMA0_START_ADDR = 0x0;	

	// 720 16bit transfers will be executed - 1440/2
	// or 360 if skip mode selected
	if(skipmode==NOSKIP) *pDMA0_X_COUNT = 720;
	else *pDMA0_X_COUNT = 360;

	// The modifier is set to 2 because of the 16bit transfers
	*pDMA0_X_MODIFY = 0x2;	
														
	// Number of Rows - active video only 240+240
	//  Note the SAA7113 can be programmed to generate 242 lines
	*pDMA0_Y_COUNT = 484;

	//The modifier is set to 2's complement of row length
	if(skipmode==NOSKIP)  *pDMA0_Y_MODIFY = 1440-1438;	
	else *pDMA0_Y_MODIFY = 720-718; 
//	*pDMA0_Y_MODIFY = 0;

	//PPI Peripheral is used
	*pDMA0_PERIPHERAL_MAP = 0x0;	
	
	//DMA Config: Enable DMA | Memory write DMA | Discard DMA FIFO before start 
	//  | enable assertation of interrupt | NDSIZE for stop mode | Autobuffer 
	*pDMA0_CONFIG = 0x0010 | DMAEN | DI_EN | WNR | WDSIZE_16 | RESTART | 0x1000;	

}

//--------------------------------------------------------------------------//
// Function:	Init_DMA_CH0_CM200											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures the PPI DMA channel to support		//
//				transfer of video input from the CM200.						//
//				Default sensor output is 1600x1200x10 bits					//
//--------------------------------------------------------------------------//
void Init_DMA_CH0_CM200()
{
	// Target address of the DMA
	*pDMA0_START_ADDR = 0x0;	

	// 1600 10bit transfers per line - progressive
	*pDMA0_X_COUNT = 1600;

	// The modifier is set to 2 because of the 16bit transfers
	*pDMA0_X_MODIFY = 0x2;	
														
	// Default number of Rows is 1200
	*pDMA0_Y_COUNT = 1200;

	//The modifier is set to 2's complement of row length
	*pDMA0_Y_MODIFY = 1600-1598;	

	//PPI Peripheral is used
	*pDMA0_PERIPHERAL_MAP = 0x0;	
	
	//DMA Config: Enable DMA | Memory write DMA | Discard DMA FIFO before start 
	//  | enable assertation of interrupt | NDSIZE for stop mode | Autobuffer 
	*pDMA0_CONFIG = 0x0010 | DMAEN | DI_EN | WNR | WDSIZE_16 | RESTART | 0x1000;	

}
//--------------------------------------------------------------------------//
// Function:	Init_MasterSPI												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	Initialize the SPI as Master with Mode 3 setting so Atmel	//
//				Flash is handled correctly.									//
//				                                                          	//
//				                                							//
//--------------------------------------------------------------------------//
void Init_MasterSPI(void)
{
	// Set baud rate to 118.8MHz/(2*26) = 2.28MHz
	*pSPI_BAUD = 0x001A;
	
	// Set Control Register
	//  TIMode=01 (Tx), SZ=1, Size=8bits, MSB first,SPE=1,MSTR=1
	//  SCK is active low - Mode 3
	*pSPI_CTL = 0x5C09;
	
	//  Set Flag Registers for PF select
	//     PF2 is Flash 0, PF3 is Flash 1
//	*pSPI_FLG = 0xFF0C;  // PF2,3 active
	*pSPI_FLG = 0xFF04;  // PF2 active
	
	// Clear RX,TX error flags
	*pSPI_STAT |= 0x0016;
	
}

//--------------------------------------------------------------------------//
// Function:	Init_PLL													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This routine sets the SCLK and CCLK clocks and system		//
//				configuration register.										//
//				For 400MHz operation from a 27MHz input clock set 			//
//				                            								//
//				Default startup is PLL_CTL=0x1400  27MHz*10=270 			//
//				                   PLL_DIV=0x0005  SCLK=54MHz   			//
//--------------------------------------------------------------------------//
void Init_PLL(void)
{
	volatile int test=0;
	
	//Initialize System Configuration Register
	sysreg_write(reg_SYSCFG, 0x32);
	
	if(0) {
		//  Set VCO multiplier to 22 MSEL - 27MHz*22 = 594MHz
		*pPLL_CTL = 0x2C00;	
		//  CSEL = 0, SSEL = 5 - CCLK = 594MHz SCLK = 594MHz/5 = 118.8MHz
		*pPLL_DIV = 0x0005;
		}
	else if(0) {
		//  Set VCO multiplier to 15 MSEL - 27MHz*15 = 405MHz
		*pPLL_CTL = 0x1E00;	
		//  CSEL = 0, SSEL = 3 - CCLK = 405MHz SCLK = 405MHz/4 = 101.25MHz
		*pPLL_DIV = 0x0004;
		}
	else if(0) {
		//  Set VCO multiplier to 14 MSEL - 27MHz*14 = 378MHz
		*pPLL_CTL = 0x1C00;	
		//  CSEL = 0, SSEL = 3 - CCLK = 378MHz SCLK = 378MHz/4 = 94.5MHz
		*pPLL_DIV = 0x0004;
		}
	else if(1) {
		//  Set VCO multiplier to 14 MSEL - 27MHz*14 = 378MHz
		*pPLL_CTL = 0x1C00;	
		//  CSEL = 0, SSEL = 3 - CCLK = 378MHz SCLK = 378MHz/3 = 126MHz
		*pPLL_DIV = 0x0003;
		}
	else if(0) {
		//  Set VCO multiplier to 29 MSEL - 27MHz*29 = 783MHz
		*pPLL_CTL = 0x3A00;	
		//  CSEL = 1, SSEL = 6 - CCLK = 391.5MHz SCLK = 783MHz/6 = 130.5MHz
		*pPLL_DIV = 0x0016;
		}
	ssync();
	idle();
}

//--------------------------------------------------------------------------//
// Function:	Init_PPI_CM64												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures the PPI for ITU-656 reception		//
//				Check the setup values for the SAA7113 for correct settings	//
//				 															//
//--------------------------------------------------------------------------//
void Init_PPI_CM64(unsigned char skipmode)
{
	short ppistatus;
	
	// The PPI is set to receive 525 lines for each frame
	*pPPI_FRAME = 525;
	
	// Reset the PPI status register by reading it
	ppistatus = *pPPI_STATUS;

	// PPI enabled, input mode, active video only, receive field 1&2, 
	// packing enabled, DMA16 enabled, skipping disabled, 8bit data bus, nothing inverted
	// If skipping enable - skip even-1 or skip odd-0
	if(skipmode==SKIPEVEN) *pPPI_CONTROL = PORT_EN | FLD_SEL | PACK_EN | SKIP_EN | SKIP_EO | DLEN_8 ;
	else if(skipmode==SKIPODD) *pPPI_CONTROL = PORT_EN | FLD_SEL | PACK_EN | SKIP_EN | DLEN_8 ;
	else if(skipmode==NOSKIP) *pPPI_CONTROL = PORT_EN | FLD_SEL | PACK_EN | DLEN_8 ;
}

//--------------------------------------------------------------------------//
// Function:	Init_PPI_CM64												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures the PPI for ICM109S reception		//
//				Check the setup values for the ICM019S for correct settings	//
//				TODO - programmable frame size								//
//--------------------------------------------------------------------------//
void Init_PPI_CM200()
{
	short ppistatus;
	
	// Set PPI width count
	*pPPI_COUNT = 1600 - 1;

	// The PPI is set to receive 1200 lines for each frame
	*pPPI_FRAME = 1200;
	
	// Reset the PPI status register by reading it
	ppistatus = *pPPI_STATUS;

	// PPI enabled, input mode,  nothing inverted,  
	// skipping disabled, 10bit data bus, 
	// non-ITU656 mode, Vsync, HSync start on raising edge
	*pPPI_CONTROL = PORT_EN | DLEN(10) | 0x00C0 ;
}

//--------------------------------------------------------------------------//
// Function:	Init_SDRAM													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function the SDRAM controller for 32MByte operation	//
//--------------------------------------------------------------------------//
void Init_SDRAM(void)
{

	// SDRAM Refresh Rate Control Register
	//   SCLK=118.8MHz, Refresh Period=64msec, No row address=8192
	//   (118.8MHz*64msec/8192)-(6+3)=919 or 397
	*pEBIU_SDRRC = 0x00000817;	

	//SDRAM Memory Bank Control Register
	*pEBIU_SDBCTL = 0x00000013;

	//SDRAM Memory Global Control Register
	//  CAS Latency - 3 cycles, RAS=6, RP=3, RCD=3
	//  WR=2, PUPSD=0, PSSE=1, SRFS=0, EDUFE=0, FBBRW=0
	//  EMREN=0, TCSR=0, CDDBG=0	
	*pEBIU_SDGCTL = 0x0091998d;	
													
}

//--------------------------------------------------------------------------//
// Function:	Init_UART													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures the UART for Interrup operation	//
//				at a specified baud rate, 8 data bits, no parity, 1 stop	//
//				Two circular buffers - one for Tx one for Rx handle incoming//
//				and outgoing characters - xon/xoff flow control support is  //
//				included.													//
//				TODO - ready the PLL registers and adjust baud rate constant//
//--------------------------------------------------------------------------//
void Init_UART(long baudrate)
{
	unsigned char inchar,x;

	// Initialize circular buffer pointers and control flags
	x = 0; 			// flush receive buffer
	rxcnt = x; 		// rx buffer empty
	xoffsent = NO;
	xonsent = NO;
	checkflag = NO;
	enablexonoff = YES; // turn serial driver xon/xoff flow control on
	echoinchars = NO;  // echo all received characters
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
		
	//  Set line control reg - 8 bits, 1 stop, no parity
	*pUART_LCR = 0x0083;
	
	//  Set baud rate
	//    SCLK = 130.5MHz/16 = 8,156,250
	//    8156250/9600 = 772
	//    8156250/57600 = 142
	if(0) {
		*pUART_DLL = (8156250/baudrate)%256;
		*pUART_DLH = (8156250/baudrate)/256;
		}
	//  If SCLK is 101.25 then 101.25/16 = 6328125
	else if(0) {
		*pUART_DLL = (6328125/baudrate)%256;
		*pUART_DLH = (6328125/baudrate)/256;
		}
	//  If SCLK is 126 then 126/16 = 787500
	else if(1) {
		*pUART_DLL = (7875000/baudrate)%256;
		*pUART_DLH = (7875000/baudrate)/256;
		}
	//  If SCLK is 94.5 then 94.5/16 = 590625
	else if(0) {
		*pUART_DLL = (5906250/baudrate)%256;
		*pUART_DLH = (5906250/baudrate)/256;
		}
	
//	*pUART_DLL = 0x00b2;
//	*pUART_DLH = 0x0001;
	
	//  Enable access to RBR,RBR and IER
	*pUART_LCR = 0x0003;

	// Clear Rx buffer register
	inchar = *pUART_RBR;
	
	// Enable UART
	*pUART_GCTL = UCEN;
	
	// Enable Tx and Rx interrupts 
	*pUART_IER = ETBEI | ERBFI;
}





