/**************************************************************

   Program Title:  P1 AVR Boot/Monitor
   
   Program Name:  P1_Monitor.c		  Version: 0.53
   
   Author:  Stephen Swift			Date:  October 10, 2004
   
   This program implements a rudimentary monitor and boot program
   for the AVR running on the P1 processor.  The Monitor controls 
   test, program load via serial and Blackfin setup.
   
   The Blackfin is held in reset until released by the Reset Blackfin
   command.  Default Blackfin boot mode is SPI Master.  AVR leaves
   SPI port disabled with all control line as inputs.
   
   Copyright 2004-2005, SOC Machines, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright (C) 2005 by SOC Machines, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P!_Monitor.c,v $
 * Revision 0.52  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
#include <iom16v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include "uart.h"

//  EEPROM Definitions
#define   MAJORVER   	 0
#define   MINORVER       53
#define   PROCESSORTYPE  70  // P1 processor
#define   ETHERNETMACID  "002078E11156"
#define   IPADDRESS      "256.128.0.24"
#define   IPMASK         "256.256.256.0"
#define   IPGATEWAY		 "256.45.25.0"
#define   USERID		 "PIRANA1"
#define   PASSWORD       "SECRET"

// Other definitions
#define  LEEPROM   'e'
#define  LAVRFLASH 'a'
#define  LFLASH    'f'
#define  BOOTMODE0 '0'
#define  BOOTMODE1 '1'
#define  BOOTMODE2 '2'
#define  BOOTMODE3 '3'
#define  NO  0x00
#define  YES 0x01
#define  READCONTINUOUS  1
#define  READBUFFER      2

//  Variable declarations
unsigned char programmode;
unsigned int bytesonline;
unsigned int inteladdress;
unsigned char crc,intelbinary[20];  // length can vary
unsigned char hexbyte[3];
unsigned char hexaddress[5];

/* Vector numbers for ATmega16 */
#pragma interrupt_handler UART_RX_Interrupt:12 UART_TX_Interrupt:13

/* USART Buffer Defines */
#define UART_RX_BUFFER_SIZE 64 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#define UART_TX_BUFFER_SIZE 64 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )

/* Static Variables */
static unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static volatile unsigned char UART_RxHead;
static volatile unsigned char UART_RxTail;
static unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
static volatile unsigned char UART_TxHead;
static volatile unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars, sendxonxoff;
char  rxcnt, checkflag;

unsigned char FlashAddress[3];
const char signon1[] = "\n\r\n\rP1 Pirana AVR Monitor V0.52\n\r";
const char signon2[] = "Copyright 2004,  SOC Machines, Inc.\n\r\n\r";
const char signon3[] = "   D[E|A|F]  - Display AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon4[] = "   P[E|A|F]  - Program AVR EERPOM, AVR Flash or Flash memory\n\r";
const char signon5[] = "   R   - Reset Blackfin - Blackfin released\n\r";
const char signon6[] = "   H   - Halt/Release Blackfin (Red LED on - Blackfin Halted)\n\r";
const char signon65[] = "   B   - Set Blackfin Bootmode [0,2,3] 3-default\n\r";
const char signon66[] = "\n\r Enter Blackfin Bootmode [0,2,3]: ";
const char signon67[] = "\n\r Blackfin Bootmode set to ";
const char signon7[] = "   V   - Version\n\r";
const char signon8[] = "   T   - Test AVR\n\r";
const char signon85[] = "   A   - Read all analog channels\n\r";
const char signon9[] = "   S   - Signon message\n\r\n\r-";
const char signon10[] = "  Note:  Blackfin held in reset until released by R command\n\r\n\r-";

const char displaymessage1[] = "\n\rDisplay EEPROM from 0 to 20\n\r";
const char displaymessage2[] = "\n\rDisplay AVR Flash from 0x0054 to 254\n\r";
const char displaymessage3[] = "\n\rDisplay SPI Flash from 0 to 100\n\r";
const char displaymessage33[] = "\n\rNo mode selected\n\r";

const char progmessage[] = "P1 Pirana AVR Monitor Version: ";

const char displaymessage4[] = "\n\rLoad EEPROM from address ";
const char displaymessage5[] = " length ";
const char displaymessage6[] = "\n\rLoad SPI Flash from address ";

const char loadmessage1[] = "\n\rLoad EEPROM\n\r";
const char loadmessage2[] = "\n\rLoad SPI Flash\n\r";
const char loadmessage3[] = "\n\rLoad AVR Flash\n\r";

//****************************************************************
//****************************************************************
// Initialization code
//****************************************************************
//****************************************************************

// Initialize UART
void InitUART( unsigned char baudrate )
{
	unsigned char x;
	
	/* set the baud rate */
	UBRRH = 0x00;
	UBRRL = baudrate; 
	/* enable UART receiver and transmitter */
	UCSRB = BIT(RXCIE)|BIT(UDRIE)|BIT(RXEN)|BIT(TXEN); 
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
	UCSRC = (1<<URSEL)|(3<<UCSZ0);

	x = 0; /* flush receive buffer */
	rxcnt = x; /* rx buffer empty */
	xoffsent = NO;
	xonsent = NO;
	checkflag = NO;
	enablexonoff = NO; /* turn serial driver xon/xoff flow control on  */
	echoinchars = NO;  /* echo all received characters */
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
}

//  Setup the Analog to Digital converter
//  TBD - convert to background interrupt process
void InitADC()
{
	DDRA = 0x00;  // All inputs
		   		  // Setup Analog TDB
}

//  Start the real time clock timed by 32.768 watch crystal
void InitRealTimeClock()
{
	//    The external clock crystal is 32,768Hz - divide by 256 
	//    yields 128.  The prescaler divides by 8, 32, 64, 128,
	//    256 or 1024.
	ASSR = (1<<AS2);
	TCCR2 = 0x01;   // No Prescaler - set overflow on 256 - 128 ticks/sec
//	TCCR2 = 0x02;   // Prescaler 8 - set overflow on 64 - 64 ticks/sec

    // setup interrupts for real time clock

}

// Setup watch dog timer - TBD
void InitWatchdog()
{
}

void Init_SPI_Master(void)
{
    // Set MOSI and SCK output, all others input
	DDRB = (1<<PB5)|(1<<PB7)|(1<<PB4);
	PORTB |= 0xA0;  // Set bits MOSI & SCK

	//  Make Flash Chip select PC4 output and set high
	DDRC |= 0x10; 
	PORTC |= 0x10;  
	
	// Enable SPI, Master, set clock rate fck
	//    Select SPI Mode 3 - CPOL=1, CPHA=1
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL)|(1<<CPHA)|(1<<SPR0);
//	SPCR = 0x91;
}

//*********************************************************
//*********************************************************
//     Interrupt handlers
//*********************************************************
//*********************************************************

//  Uart serial receiver interrupt handler
//  Flow control by XON/XOFF
void UART_RX_Interrupt( void )
{
	unsigned char data;
	unsigned char tmphead;
	
	data = UDR; /* read the received data */
	/* calculate buffer index */
	tmphead = (UART_RxHead + 1) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead; /* store new index */
	UART_RxBuf[tmphead] = data; /* store received data in buffer */
	
	rxcnt++;
	if(rxcnt>0x3f) checkflag = YES;
	if((rxcnt>(UART_RX_BUFFER_SIZE-20))&&(xoffsent==NO)&&(enablexonoff==YES)) {
	/* If tx buffer is empty then transmit xoff else signal tx to send xoff */
	    sendxonxoff = YES;
		UCSRB |= (1<<RXCIE); /* enable RXCIE interrupt */
//		if(UART_TxHead == UART_TxTail)  {
//			UDR = XOFF;
//			xoffsent = YES;
//			}		
		}
//	if ( tmphead == UART_RxTail )
//		{
//		/* ERROR! Receive buffer overflow */
//		}
}

//  UART transmit interrupt handler
//  Flow control by XON/XOFF
void UART_TX_Interrupt( void )
{
	unsigned char tmptail;

	/* check if all data is transmitted */
	if(UART_TxHead != UART_TxTail)  {
	    /* check chars in rx buffer - if too many trans XOFF */
	    if(sendxonxoff == YES)  {
			 UDR = XOFF;
//			 UDR = '%';
			 xoffsent = YES;
			 sendxonxoff = NO;
			 }
		else {
			 /* calculate buffer index */
			 tmptail = (UART_TxTail + 1) & UART_TX_BUFFER_MASK;
			 UART_TxTail = tmptail; /* store new index */
			 UDR = UART_TxBuf[tmptail]; /* start transmition */
			 if(UART_TxBuf[tmptail]==XON) {
			 	  xoffsent = NO;
				  xonsent = NO;
				  }
			 }
		}
	else  {  /* check chars in rx buffer - if too many trans XOFF */
		if(sendxonxoff == YES)  {
			 UDR = XOFF;
//			 UDR = '$';
			 xoffsent = YES;
			 sendxonxoff = NO;
			 }
		else UCSRB &= ~(1<<UDRIE); /* disable UDRE interrupt */
		}
	}

//*********************************************************
//*********************************************************
//      Utilities
//*********************************************************
//*********************************************************
	
void putchar(unsigned char data)
{
	unsigned char tmphead;
	
	/* calculate buffer index */
	tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
	/* wait for free space in buffer */
	while (tmphead == UART_TxTail) ;
	
	UART_TxBuf[tmphead] = data; /* store data in buffer */
	UART_TxHead = tmphead; /* store new index */
	UCSRB |= (1<<UDRIE); /* enable UDRIE interrupt */
}

// Read and write functions
unsigned char getcharnw( void )
{
	unsigned char tmptail;
	char rxtmp;

/*	while ( UART_RxHead == UART_RxTail ) ; */ /* wait for incomming data */
	if(UART_RxHead==UART_RxTail) return 0; /* nothing to get */
		
	tmptail = (UART_RxTail + 1) & UART_RX_BUFFER_MASK;/* calculate buffer index */
	UART_RxTail = tmptail; /* store new index */
	
	_CLI();
	rxcnt--;
	rxtmp = rxcnt;
	_SEI(); /* Enable interrupts => enable UART interrupts */

	if(checkflag == YES)  {
		putchar('@');	
		checkflag = NO;
		}
	if(rxtmp < 0) putchar('<');
	if(rxtmp > 0x3f) putchar('>');
	/* Allow tx at remote serial device again - buffer has more space */
	if((rxtmp<(UART_RX_BUFFER_SIZE-60))&&(xoffsent==YES)&&(enablexonoff==YES)) {
		if(xonsent==NO);  {
			putchar(XON); putchar('$');
			xonsent = YES;
			}
//		xoffsent = NO;
		}
	return UART_RxBuf[tmptail]; /* return data */
}

// Getchar waits until a valid character is received
unsigned char getchar()
{
 	while(!getcharnw()) ;
}

/* Initialize UART */
void InittUART( unsigned char baudrate )
{
 	/* set the baud rate */
	UBRRH = 0x00;
	UBRRL = baudrate; 
	UCSRB = BIT(RXEN) | BIT(TXEN); /* enable UART receiver and transmitter */
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
	UCSRC = (1<<URSEL)|(3<<UCSZ0);
}

// Polled receive function - non-wait
unsigned char ReceiveByte( void )
{
	if((UCSRA & (1<<RXC))) return UDR;
	else return 0x00;
}

// Polled transmit function
void TransmitByte( unsigned char data )
{
	while ( !(UCSRA & (1<<UDRE)) )
		; /* wait for empty transmit buffer */
	UDR = data; /* start transmission */
}

//  Buffered write to UART - wait until transmitted
void puttchar(unsigned char data)
{
 	 TransmitByte(data);
}

//  Buffered character read from UART - don't return if 
//  character is Null
unsigned char gettchar(void)
{
     unsigned char inchar;
	 
 	 while(!(inchar=ReceiveByte())) ;
 	 return inchar;
}

//  Reset the Blackfin for a short period
void ShortBlackfinReset()
{
 	 PORTD &= 0xF7;
 	 Delay(10);
	 PORTD |= 0x08;
}

//  Free SPI pins
void Release_SPI()
{
	// Disable SPI, Master 
	SPCR = 0x00;

 	// Set MOSI and SCK input, all others input
	DDRB = 0x00;
	
	// Change Atmel Flash CS to input (PF2 - Port C bit 4)
	DDRC &= 0xEF;
}

// TODO - break if no reception
void SPI_MasterTransmit(unsigned char cData)
{
    long cnt;
	
	cnt = 0;
    // Start transmisstion
	SPDR = cData;
	// Wait for transmission to complete
//	puts("wait for spi tx to complete\n\r");
	while(!(SPSR & 0x80)) ;
}

// TODO - break if no reception
unsigned char SPI_MasterReceive(void)
{
    // Wait for reception to complete
	SPDR = 0x00;
	while(!(SPSR & 0x80)) ;
	
	return SPDR;
}

// Halt Blackfin - Reset line is on Port D pin 3
void HaltBlackfin()
{
 	RED_LED_On();
	PORTD &= 0xF7;
	puts("Halt Blackfin\n\r");
}

void ReleaseBlackfin()
{
 	RED_LED_Off();
	PORTD |= 0x04;
	puts("Release Blackfin\n\r");
}

// Blackfin Reset line is on Port D pin 3
void ResetBlackfin()
{
 	RED_LED_On();
	PORTD &= 0xF7;
	LongDelay(1);
	PORTD |= 0x08;
	RED_LED_Off();
	puts("Blackfin Reset and enabled\n\r");
}

void Delay()
{
	unsigned char a, b;

	for (a = 1; a; a++)
		for (b = 1; b; b++)
			;
}

void shortdelay(int cnt)
{
 	unsigned char inbyte;
	int i;
	
    for(i=0; i<cnt; i++)  {
	    inbyte = 0x55;
		}
}

void LongDelay(int cnt)
{
 	 while(cnt > 0) {
	 	 Delay();
		 cnt--;
		 }
}

void GREEN_LED_On()
{
	PORTD &= ~BIT(6);	// low output to turn LED on
}

void GREEN_LED_Off()
{
	PORTD |= BIT(6);	// low output to turn LED on
}

void RED_LED_On()
{
 	 PORTD &= ~BIT(7);	
}

void RED_LED_Off()
{
 	 PORTD |= BIT(7);	
}

void BlinkGreen(int period)
{
 	 GREEN_LED_On();
	 LongDelay(period);
	 GREEN_LED_Off();
	 LongDelay(period);
}

void BlinkRed(int period)
{
 	 RED_LED_On();
	 LongDelay(period);
	 RED_LED_Off();
	 LongDelay(period);
}

//  Morse code for "PIRANA"
//  OK not very useful but different
void PiranaMorse()
{
 	 LongDelay(10);
 	 BlinkRed(1); BlinkRed(3); BlinkRed(3); BlinkRed(1);  //  P
	 BlinkRed(1); BlinkRed(1);  //  I
 	 BlinkRed(1); BlinkRed(3); BlinkRed(1);  //  R
 	 BlinkRed(1); BlinkRed(3);   //  A
 	 BlinkRed(3); BlinkRed(1);  //  N
 	 BlinkRed(1); BlinkRed(3);  //  A

	 LongDelay(6);
}

//  Convert binary value to ASCII int
void printint(int intvalue,char charcnt,char leadchartype)
{
 	 char outvalue[6],nolead;
 	 int  i,tenthousands, thousands, hundreds, tens;
	 
	 if(intvalue >= 0) putchar(' ');
	 else {
	 	  putchar('-');
	 	  intvalue *= -1;
		  }
	 tenthousands = intvalue/10000;
	 intvalue -= (tenthousands*10000);
	 thousands = intvalue/1000;
	 intvalue -= (thousands*1000);
	 hundreds = intvalue/100;
	 intvalue -= (hundreds*100);
	 tens = intvalue/10;
	 nolead = 0x00;
	 // TODO - suppress only leading zero's
	 if(tenthousands)  {outvalue[0] = '0' + tenthousands; nolead=0x01;}
	 else outvalue[0] = leadchartype;
	 if(thousands|nolead) {outvalue[1] = '0' + thousands;nolead=0x01;}
	 else outvalue[1] = leadchartype;
	 if(hundreds|nolead) {outvalue[2] = '0' + hundreds;nolead=0x01;}
	 else outvalue[2] = leadchartype;
	 if(tens|nolead) outvalue[3] = '0' + tens;
	 else outvalue[3] = leadchartype;
	 outvalue[4] = '0' + intvalue%10; 
	 outvalue[5] = 0x00;
	 for(i=0; i<charcnt; i++) putchar(outvalue[(5-charcnt)+i]);
}

void puts(char *outstrg)
{
 	 int i;
	 
	 for(i=0; i<strlen(outstrg); i++)  putchar(outstrg[i]);
}

void putCRLF()
{
 	 puts("\n\r");
}

// Select Blackfin boot mode - 00, 01, 10 or 11
//
// Blackfin Rev 0.3 silicon boots from Atmel SPI Flash as Boot Mode 3.
// Note: Rev 0.2 silicon can not communicate with Atmel SPI Flash
//       devices AT45DB041, AT45DB081 or ATDB45161 - must use SPI Flash
//       that supports read command 0x03 such as the Microship 25LC640
//       or ST M25P80.
void BlackfinBootMode(unsigned char bootmode)
{
 	 //  Boot mode 0 - parallel flash
 	 if(bootmode == BOOTMODE0) PORTD &= 0xCF;
	 
	 //  Blackfin SPI Slave to AVR
	 else if(bootmode == BOOTMODE2) {
	     PORTD &= 0xDF;
		 PORTD |= 0x10;
		 }
	 //  Boot from SPI Slave Flash
	 //     Rev 0.2 silicon uses read command 0x03 
	 //     Rev 0.3 silicon attempts 0x03 read then switch to Atmel SPI
	 else if(bootmode == BOOTMODE3) PORTD |= 0x30;
}


//  TBD
void EEPROMSetup()
{
 	
}

//  Access contents of AVR flash at specified address
unsigned char ReadAVRFlash(int address)
{
 	unsigned char flashbyte,rbyte;
	
 	asm("mov r30,r16 \n"
       "mov r31,r17 \n"
	   "lpm r0,Z \n"
	   "mov %flashbyte,r0 \n"); 
	rbyte = flashbyte;  
	return(rbyte);                              
}

//  Load string constant from Flash (conserves RAM)
void putfs(const char *signon1)
{ 	
	char inchar;
    int i;
	
	i = 0;
	while((inchar=*(signon1+i++))!=0x00)  putchar(inchar);
}

//  Output sign on message loaded from Flash
void signon()
{
	putfs(signon1);
	putfs(signon2);
	putfs(signon3);
	putfs(signon4);
	putfs(signon5);
	putfs(signon6);
	putfs(signon65);
	putfs(signon7);
	putfs(signon8);
	putfs(signon85);
	putfs(signon9);
//	putfs(signon10);
}

//  Convert byte to ASCII hex
void bytetohex(unsigned char byte)
{
    unsigned char inbyte,inchar;
	
	inbyte = byte & 0x0f;
	if(inbyte > 9) hexbyte[1] = (inbyte - 9) + 0x40;
	else hexbyte[1] = 0x30 + inbyte;
	inbyte = byte & 0xf0;
	inbyte >>= 4;
	if(inbyte >9) hexbyte[0] = (inbyte - 9) + 0x40;
	else hexbyte[0] = 0x30 + inbyte;
    hexbyte[2] = 0x00;
}

// Convert word to ASCII Hex
void addresstohex(int address)
{
 	 unsigned char lowbyte,highbyte;
	 int intbyte;
	 
	 lowbyte = address & 0x00FF;
	 bytetohex(lowbyte);
	 hexaddress[2] = hexbyte[0];
	 hexaddress[3] = hexbyte[1];
	 intbyte = address;
	 highbyte = address/256;
	 bytetohex(highbyte);
	 hexaddress[0] = hexbyte[0];
	 hexaddress[1] = hexbyte[1];
	 hexaddress[4] = 0x00;
}

//  Display AVR EEPROM memory
void DisplayEEPROM(int address,int numbytes)
{
 	 unsigned char i,byte,bytesonlastline,ascii[16],asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	     crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonline);
		 crcvalue += bytesonline;
		 puts(hexbyte);
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     addresstohex(newaddress);
	     puts(hexaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = EEPROMread(i+newaddress);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
		 bytetohex((~crcvalue)+0x01);
	 	 puts(hexbyte);
	     puts("  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     puts("\n\r");  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonlastline);
		 crcvalue += bytesonlastline;
		 puts(hexbyte);
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     addresstohex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = EEPROMread(i+address+linecnt*16);
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 // Output CRC
		 bytetohex((~crcvalue)+0x01);
	 	 puts(hexbyte);
	     puts("  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     putCRLF();  
		 }
}

//  Display P1 AVR Flash memory
void DisplayAVRFlash(int address,int numbytes)
{
 	 unsigned char i,byte,bytesonlastline,ascii[16],asciicnt;
	 int linecnt,numlines;

	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonline);
		 puts(hexbyte);
	     //  Output hex start address
	     addresstohex(address+linecnt*16);
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
	     puts("xx  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonlastline);
		 puts(hexbyte);
	     //  Output hex start address
	     addresstohex(address+linecnt*16);
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = ReadAVRFlash(address+i+linecnt*16);
			 ascii[asciicnt++] = byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 // Output CRC
	     puts("xx  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     putCRLF();  
		 }
}

//  SPI Flash device address construction function
//  Read address consists of 4 zero bits+11 page bits and 9 byte start
void ConvertFlashAddress(int address)
{
 	int pages,bytestart,superpages;
	
	pages = address/264;
	superpages = pages/7;
	bytestart = address - pages*264;
    FlashAddress[0] = superpages;
	FlashAddress[1] = ((pages&0x00ff)<<1) + (bytestart>>8);
	FlashAddress[2] = bytestart & 0x00FF;
}

//  SPI Flash continuous read command for AT45DB041B
//  The AT45DB041B has 2048 264byte pages
void SPI_StartReadFlashPage(int address)
{
 	 int i;
	 
	 // Send Continuous read command
 	 SPI_MasterTransmit(0x68);
//	 puts("Send read command\n\r");
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
//	 puts("Address\n\r");
	 
	 //  32 dummy bits
	 for(i=0; i<6; i++)  SPI_MasterTransmit(0x00);
//	 puts("Send dummy bits\n\r");
}

//  SPI Flash Buffer Read command for AT45DB041B
void SPI_ReadFlashBuffer(int address,int buffernumber)
{
 	 int i;
	 
	 // Send buffer read command
	 if(buffernumber==1) SPI_MasterTransmit(0x54);
	 else SPI_MasterTransmit(0x56);

//	 puts("Send read command\n\r");
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
//	 puts("Address\n\r");
	 
	 //  8 dummy bits
	 SPI_MasterTransmit(0x00);
//	 puts("Send dummy bits\n\r");
}


//  Display P1 Blackfin Flash memory
//    Blackfin must be reset
void DisplayFlash(int address,int numbytes,char readmode)
{
 	 unsigned char i,byte,bytesonlastline,ascii[16],asciicnt;
	 int linecnt,numlines;

	 //  Set PC4 low - Flash CS
	 PORTC &= 0xEF;
//	 puts("CS Low\n\r");

	 if(readmode==READCONTINUOUS) SPI_StartReadFlashPage(address);
	 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(address,1);
//	 puts("Send Continuous Read command\n\r");
	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonline);
		 puts(hexbyte);
	     //  Output hex start address
	     addresstohex(address+linecnt*16);
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = SPI_MasterReceive();
			 ascii[asciicnt++] = byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
	     puts("xx  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonlastline);
		 puts(hexbyte);
	     //  Output hex start address
	     addresstohex(address+linecnt*16);
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = SPI_MasterReceive();
			 ascii[asciicnt++] = byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 // Output CRC
	     puts("xx  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
		 putCRLF();
		 
		 // Set PC4 high - deselect Flash chip
		 PORTC |= 0x10;
//		 puts("CS High\n\r");
		 }
}

void DisplayMemory()
{
    unsigned char inchar;
	
	inchar=getchar();
	putchar(inchar);
	switch (inchar) {
	    case 'e': case 'E':
		    putfs(displaymessage1);
			DisplayEEPROM(0,20);
			
			// Only added for debug purposes - Rev A PCB has no pull up
			Release_SPI();
		    break;
		case 'a': case 'A':
		    putfs(displaymessage2);
		    DisplayAVRFlash(0x0054,200);
			
			// Only added for debug purposes - Rev A PCB has no pull up
			Release_SPI();
		    break;
		//  In order to access Blackfin Flash the AVR must ask the Blackfin
		//  to release control of the SPI lines.  This is simulated in Rev A
		//  boards by resetting the Blackfin (which should release the SPI lines)
		//  On completion the AVR SPI lines are released.
		case 'f': case 'F':
		    ShortBlackfinReset();
//			HaltBlackfin();
			Init_SPI_Master();
		    putfs(displaymessage3);
		    DisplayFlash(0x0000,100,READBUFFER);
//			Release_SPI();
//			ReleaseBlackfin();
		    break;
		default:
		    putfs(displaymessage33);
		    break;
	    }
}

//  Display AVR EEPROM memory
void DebugDisplayIntelHex()
{
 	 unsigned char i,byte,bytesonlastline,ascii[16],asciicnt;
	 int linecnt,numlines;

	 linecnt = asciicnt = 0;	 
	 if(1) {
	     putchar(':');
		 //  Output bytes on line
		 bytetohex(bytesonline);
		 puts(hexbyte);
	     //  Output hex start address
	     addresstohex(inteladdress);
	     puts(hexaddress);
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = intelbinary[i];
			 ascii[asciicnt++] = byte;
	         bytetohex(byte);
	 	     puts(hexbyte);
	         }
		 // Output CRC
	     puts("xx  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     putCRLF();  
		 }
}

// Convert ASCII hex to binary
unsigned char converttohex(char *instrg)
{
 	 unsigned char lownibble, highnibble;
	 
	 if((instrg[0] >= 'A')&(instrg[0]<='F')) highnibble = instrg[0] - 'A' + 10;
	 if((instrg[0] >= 'a')&(instrg[0]<='f')) highnibble = instrg[0] - 'a' + 10;
	 else highnibble = instrg[0] - '0';
	 
	 if((instrg[1] >= 'A')&(instrg[1]<='F')) lownibble = instrg[1] - 'A' + 10;
	 else if((instrg[1] >= 'a')&(instrg[1]<='f')) lownibble = instrg[1] - 'a' + 10;
	 else lownibble = instrg[1] - '0';
	 
	 highnibble = highnibble<<4;
	 return highnibble + lownibble;
}

// Check intelbinary CRC 
int checkcrc()
{
 	unsigned char crcbyte,crc[3];
	
	crc[0] = getchar();
	crc[1] = getchar();
	crc[2] = 0;
 	return 1;
}

// Convert Intel Hex ASCII string to binary
// :10 0100 00 0C948F0D0C941D0E0C941D0E0C941D0E 38
// :1001000000C941D0E0C941D0E0C944E590C941D0E38
// This routine decodes a line of Intel Hex.  Other 
// routines write the data to either AVR Flash, external Flash
// or AVR EEPROM

// TODO - end of file line :000000000FF - ignore
int DecodeIntelHex()
{
 	 unsigned char strg[8],addressbyte,inbyte;
	 int i;
	 
	 // See if first char is start of Intel hex format
//	 if(instrg[0] != ':')  return -1;
	 
	 // Convert count of bytes on line
	 strg[0] = getchar();
	 strg[1] = getchar();
	 strg[2] = 0;
	 bytesonline = converttohex(strg);
	 
	 // Convert address to binary - ignore filler byte
	 // Typical Intel Hex consists of 4 ASCII characters
	 inteladdress = 0;
	 for(i=0; i<2; i++) {
	     strg[0] = getchar();
		 strg[1] = getchar();
	     strg[2] = 0x00;
		 addressbyte = converttohex(strg);
		 if(i==1) inteladdress += addressbyte;
//		 else if(i==2) inteladdress += addressbyte*65535;
		 else inteladdress = addressbyte*256;
		 }
	 
	 // Read dump byte (2 characters)
	 strg[0] = getchar();
	 strg[1] = getchar();
	 
	 // Convert all data to binary
	 for(i=0; i<bytesonline; i++) {
	     strg[0] = getchar();
		 strg[1] = getchar();
		 strg[2] = 0x00;
		 inbyte = converttohex(strg);
		 intelbinary[i] = inbyte;
	 	 }
		 
	// Check CRC - todo
	if(checkcrc()) return 1;
	else return -1;
}

//  Write decoded Intel hex to EEPROM
void ProgramEEPROM()
{
     unsigned char i;
	 
	 for(i=0; i<bytesonline; i++)  EEPROMwrite(inteladdress+i,intelbinary[i]);
}

//  TBD
void ProgramAVRFlash()
{
}

//  SPI Flash writer
void ProgramFlash()
{
     unsigned char i;
	 
	 // Flash CS low
	 PORTC &= 0xEF;   
	 
	 //  Buffer write
	 SPI_MasterTransmit(0x84);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
//	 while(1) {
//     intelbinary[0] = 0x00;
//	 intelbinary[1] = 0x01;
//	 intelbinary[2] = 0x02;
//	 intelbinary[3] = 0x03;
	 for(i=0; i<bytesonline; i++)  SPI_MasterTransmit(intelbinary[i]);
//	 }

	 // Flash CS high
	 PORTC |= 0x10;   
	 DebugDisplayIntelHex();
}

//  Main memory programming selection function
//   Program EEPROM, AVR Flash or SPI Flash
void ProgramMemory()
{
 	DecodeIntelHex();
	switch (programmode) {
	    case LEEPROM:
		    putfs(displaymessage4);
			printint(inteladdress,4,' ');
			putfs(displaymessage5);
			printint(bytesonline,3,' ');
			putCRLF();
		    ProgramEEPROM();
//			DebugDisplayEEPROM();
			break;
		case LAVRFLASH:
		    ProgramAVRFlash();
			break;
		//  In order to access Blackfin Flash the Blackfin must not be
		//  using the SPI lines.  The Blackfin is asked to release the SPI
		//  lines before programming the Flash.
		case LFLASH:
		    ShortBlackfinReset();
//			HaltBlackfin();
			Init_SPI_Master();
		    putfs(displaymessage6);
			printint(inteladdress,4,' ');
			putfs(displaymessage5);
			printint(bytesonline,3,' ');
			putCRLF();
		    ProgramFlash();
			Release_SPI();
//			ReleaseBlackfin();
			break;
		} 
}

//  
void LoadMemory()
{
    unsigned char inchar;
	
	inchar = getchar();
	putchar(inchar);
	switch (inchar) {
	    case 'e': case 'E':
			programmode = LEEPROM;
			putfs(loadmessage1);
		    break;
		case 'f': case 'F':
			programmode = LFLASH;
			putfs(loadmessage2);
		    break;
		case 'a': case 'A':
			programmode = LAVRFLASH;
		    putfs(loadmessage3);
		    break;
		default:
		   break;
	    }
}

// Write a string of characters to EEPROM at specified start location
void WriteEEPROM(int location, unsigned char *strg)
{
 	 int i;
	 
	 i = 0;
	 while(strg[i] != 0x00)  {
	     EEPROMwrite(location+i,strg[i]);
		 i++;
		 }
}

//  Output program version
//   TBD - read from EEPROM
void ProgramVersion()
{
      char version[5]="0.53";
	  
	  putfs(progmessage);
	  puts(version);
	  putCRLF();
}

//  Read selected analog channel - wait until it completes
//   TBD - background interrupt process
int AnalogCh(int ch)
{
 	unsigned char low,high;
	
	
	if(ch==0) ADMUX = 0x40;  // Select AVCC, ch 0
	else if(ch==1) ADMUX = 0x41; // Select AVCC, ch 0
	else if(ch==2) ADMUX = 0x42;
	else if(ch==3) ADMUX = 0x43;
	else if(ch==4) ADMUX = 0x44;
	else if(ch==5) ADMUX = 0x45;
	else if(ch==6) ADMUX = 0x46;
	else if(ch==7) ADMUX = 0x47;
	
	ADCSRA = 0xC1; // Enable ADC, start conversion, max clock
	
	while(!(ADCSRA&0x10)) ; // loop until conversion completes
	
	low = ADCL;
	high = ADCH;
 	return (low+high*256);
}

void DumpAnalogChs()
{
    int  i,analogvalue;
	
	puts("Analog Channels 0..7\n\r");
 	for(i=0; i<8; i++) {
	    analogvalue = AnalogCh(i);
		printint(analogvalue,4,' ');
		} 
	puts("\n\r"); 
}

//  Convert ASCII ip address to binary - TBD
void convertiptobinary(unsigned char *ip)
{
}

unsigned char  ip[5],ipmask[5],ipgate[5];

//  Get set Internet settings
void InternetSettings()
{
	convertiptobinary(IPADDRESS);
}

//  Simple test routine
void TestAVR()
{
 	 char i,j;
	 
 	 puts("\n\rTesting AVR Version 0.34\n\r");
//	 TestAVRRAM();
//	 TestAVREEPROM();
//	 TestAVRFLASH();
//	 TestBoardFLASH():
	 j = 0;
	 putchar(' ');
	 for(i=0x20; i<0x7f; i++) {
	 	 putchar(i);
		 if(j++ == 60)  {
		     j = 0;
			 puts("\n\r ");
			 }
		 }
	 puts("\n\r ");
	 DumpAnalogChs();
}


//  Set Blackfin boot mode
void SetBlackfinBootMode()
{
 	unsigned char bootmode;
	
    putfs(signon66);  // Ask for boot mode
	bootmode = getchar();
	putchar(bootmode);
	putfs(signon67);
	putchar(bootmode);
	puts("\n\r");
	BlackfinBootMode(bootmode);
}

//************************************************************
//  Initialize the AVR - set port conditions for Blackfin
//************************************************************
void InitAVR()
{
    // Setup constants and variables
	programmode = 'o';  // no programming mode selected
	
    // Setup Port A
	InitADC();
	
	// Setup Port B - SPI + General digital IO
	//   Setup SPI 
//    Init_SPI_Master();
	
	// Setup Port C - PF2,0 control + RS-485 + 2-Wire
	DDRC = 0x08;   // PC3 - RS-485 output and PC4 - PF2 Flash CS input
	PORTC = 0x18;  // Set PC4,3 high - deselect Flash CS
	
	//    Turn on real time clock on PC7,6 Timer/Counter 2
	InitRealTimeClock();
	
	//    Setup 2-Wire Interface
	TWBR = 0x10;   // SCL freq = sclk/(16+2*TWBR)*4*TWPS
	//  This needs additional investigation
	
	// Setup Port D  - Blackfin control lines
	// Note ResetB also controls the Flash chip
	DDRD = 0xFF;	// output
	PORTD = 0x00;	// all off
	
	// Set initial Blackfin conditions - Boot mode 
	// If BMODE=00 then the Blackfin doesn't control SPI - this may work
	// BMODE1 is PD5
	// BMODE0 is PD4
	BlackfinBootMode(BOOTMODE3);
	// RESETB is PD3 - Release ResetB
//	Delay(10);
//	PORTD |= 0x08;
	
	// Turn LEDs off
	RED_LED_Off();
	GREEN_LED_Off();
	
	// Watchdog timer setup
	InitWatchdog();
	
	// Initialize UART - Interrupt
	InitUART(11); /* set the baudrate to 9,600 bps using a
					2MHz clock rate */
	_SEI(); /* Enable interrupts => enable UART interrupts */

}

//*************************************************************
//  Main work start here
//*************************************************************
void main()
{
 	char toggle;
	unsigned char inchar;
	int i,j,loopmode,blackfinresetmode;
	long loopcnt,loopsetting;
	
	InitAVR();
	
	loopcnt = 0; loopmode = 0;  toggle = 0;
	loopsetting = 5000;

	BlinkRed(2);
	BlinkGreen(2);
	signon();
	blackfinresetmode = 0x00;
//	WriteEEPROM(0,"SOC Machines V0.51");

	// Reset and release Blackfin
	Release_SPI();
	ResetBlackfin();
	putchar('-');
	
	while (1) {
		  //  Test case 1
		  if(0) {
		  		PiranaMorse();
				}
		  //  Test case 2
		  else if(0) {
		  		GREEN_LED_On();
				Delay();
				RED_LED_On();
				Delay();
				GREEN_LED_Off();
				Delay();
				RED_LED_On();
				Delay();
				}
		  // real work starts here
		  else if(1) {
			   if((inchar = getcharnw())) {
			   		switch (inchar) {
					    case 's': case 'S':
						    signon();
							break;
					    case 'd': case 'D':
						    putchar(inchar);
						    DisplayMemory();
							putchar('-');
							break;
					    case 'p': case 'P':
							putchar(inchar);
						    LoadMemory();
							putchar('-');
							break;
					    case 'a': case 'A':
						    DumpAnalogChs();
							putchar('-');
							break;
					    case 'v': case 'V':
						    ProgramVersion();
							putchar('-');
							break;
					    case 'r': case 'R':
							Release_SPI();
						    ResetBlackfin();
							putchar('-');
							break;
					    case 'b': case 'B':
							putchar(inchar);
							SetBlackfinBootMode();
							putchar('-');
							break;
					    case 'h': case 'H':
							if(blackfinresetmode==0x01) {
								 ReleaseBlackfin();
								 blackfinresetmode = 0x00;
								 }
							else {
								 HaltBlackfin();
								 blackfinresetmode = 0x01;
								 }
							putchar('-');
							break;
					    case ':':
						    ProgramMemory();
							putchar('-');
							break;
						case 'i': case 'I':
						    InternetSettings();
							putchar('-');
							break;
						case 'l': case 'L':
						    while(!(inchar=getcharnw())) putchar(0x55);
							putCRLF();
							putchar('-');
							break;
						case 't': case 'T':
						    while(!(inchar=getcharnw()))  TestAVR();
							signon();
							break;
						case 'u':
							loopsetting += 1000;
							break;
						case 'g':
							loopsetting -= 1000;
							break;
						default:
							putchar(inchar);
							break;
						}
			   		
			   		} 
//			   LongDelay(10);
			   // Blink green led slowly to show AVR is alive
			   if(loopcnt++ > loopsetting) {
			   		loopcnt = 0;
					if(loopmode == 1)  {
					    GREEN_LED_On();
						loopmode = 0;
						}
					else {
						GREEN_LED_Off();
						loopmode = 1;
						}
			   	 	}
//			   LongDelay(10);
		  	   }
		}
	}
