// *******************************************************
//
//   Program:  I2C.c        Version: 1.1 
//
//   Originally written By P. Fletcher-Jones on 16/11/01
//   
//   Modified to work on the P1 Blackfin processor
//   Modifications Copyright 2004, SOC Machines, Inc. 
//
//   The I2C functionality is used to control two PF lines
//   of the Blackfin PF port.  These lines are used to 
//   communicate via video input adapters connected to the 
//   P1 video port (connector J3, J17).
//
//   PF assignments are as follows:
//
//      PF0 - SDA   - data line
//      PF1 - SCL   - clock line
//
//   SDA (PF0) is a bi-directional data line.
//
// *******************************************************
/*
 * Copyright (C) 2005-2006 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_I2C.h,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
// ****************************** //
// *** I2C Hardware Interface *** //
// ****************************** //

//  Need to assign SDA (PF0) and SCL (PF1) to SDA and SCL
//  SDA is PF0, SCL is PF1
#define  SDA  0x0003
#define  SCL  0x0003

#define I2C_100KHZ  4
#define I2C_400KHZ  1
#define I2C_PERIOD  2

//  SAA7113 Slave address if RTS0 high immediately after Reset release
//#define   SAA7113_Write_ADD  0x25
//#define   SAA7113_Read_ADD   0x26

#define   SAA7113_Write_ADD  0x4A
#define   SAA7113_Read_ADD   0x4B

//#define   SAA7113_Write_ADD  0x48
//#define   SAA7113_Read_ADD   0x49

#define   WRITECOMMAND  1
#define   READCOMMAND   2

#define   SKIPEVEN		1
#define	  SKIPODD		2
#define	  NOSKIP		3

//  Function definitions
void Set_SDA(char);
void Set_SCL(char);
void Init_I2C(void);
void Pulse_SCL(void);
void I2C_Start (void);
void I2C_Stop (void);
unsigned char I2C_Ackn(void);
int Write_I2C_Control(unsigned char);
int Write_I2C_Byte(unsigned char);
unsigned char Read_I2C_Byte(void);
int Write_CM64(unsigned char, unsigned char);
unsigned char Read_CM64(unsigned char);
int Write_CM200(unsigned char, unsigned char);
unsigned char Read_CM200(unsigned char);
void Reset_SAA7113(void);
void Reset_CM200(void);
void Init_SAA7113(short);
void Init_CM64(short,unsigned char,long);
void Halt_VideoDMAPPI(void);
void Init_ICM109S(void);


