// **************************************************************************
//
//  File:  P1_ISRs.c				Version: 0.51
//
//  Date:  October 20, 2004		
//
//  Interrupt service routines for P1 Blackfin
//
//  Copyright 2004-2008, SOC Robotics, Inc.
//
// **************************************************************************
/*
 * Copyright (C) 2005-2008 by SOC Robotics, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-machines.com
 */

/*
 * $Log: P1_ISRs.c,v $
 * Revision 0.54  2005/06/04 18:36:42  steve
 * Preview release
 *
 */
 //#include <cdefBF532.h>
#include <sys\exception.h>		//Interrupt Handling Header
#include "P1_Main.h"
#include "P1_Extern.h"

//--------------------------------------------------------------------------//
// Function:	PPI_DMA_CH0													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time Timer0 expires.				//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(DMA0_PPI_ISR)	// <--|declaration in  exception.h -->
{									//    |declaration with _pragma(interrupt) the ISR Startaddress 


	// Disable the interrupt request
	*pDMA0_IRQ_STATUS = 0x1;	
	
	puts( "\nThe DMA0 PPI Interrupt has been entered!\n" );

}//end DMA0_PPI_ISR


//--------------------------------------------------------------------------//
// Function:	Timer0_ISR													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time Timer0 expires.				//
//				The old LED pattern is shifted by one; the direction		//
//				depends on the state of sLight_Move_Direction, which is		//
//				changed in FlagA_ISR.										//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(Timer0_ISR)
{
	static unsigned char ucActive_LED = 0x01;

	// confirm interrupt handling
	*pTIMER_STATUS = 0x0001;

}

//--------------------------------------------------------------------------//
// Function:	UART_TX_ISR													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time the UART TX buffer becomes	//
//				empty.  The ISR supports a circular TX buffer with xon/xoff	//
//				flow control. If no characters to transmit turn Ints off	//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(UART_TX_ISR)
{
	unsigned char tmptail;

	// Check if all data is transmitted
	if(UART_TxHead != UART_TxTail)  {
	    // check chars in rx buffer - if too many trans XOFF
	    if(sendxonxoff == YES)  {
			 *pUART_THR = XOFF;
//			 UDR = '%';
			 xoffsent = YES;
			 sendxonxoff = NO;
			 }
		else {
			 // Calculate buffer index			 
			 tmptail = UART_TxTail + 1;
			 if(tmptail>=UART_TX_BUFFER_SIZE) tmptail = 0;
			 UART_TxTail = tmptail; 	// store new index
			 *pUART_THR = UART_TxBuf[tmptail]; // start transmission
			 if(UART_TxBuf[tmptail]==XON) {
			 	  xoffsent = NO;
				  xonsent = NO;
				  }
			 }
		}
	// check chars in rx buffer - if too many trans XOFF
	else  {  
		if(sendxonxoff == YES)  {
			 *pUART_THR = XOFF;
//			 UDR = '$';
			 xoffsent = YES;
			 sendxonxoff = NO;
			 }
		// disable ETBEI interrupt
		else {
			*pUART_IER &= 0xFFFD;
		//	*pSIC_IMASK = 0x0000C000;
			}
		}
}

//--------------------------------------------------------------------------//
// Function:	UART_RX_ISR													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This ISR is executed every time a character is received into//
//				the rx buffer. The ISR supports a circular RX buffer with   //
//				xon/xoff flow control.												//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(UART_RX_ISR)
{
	unsigned char data;
	unsigned char tmphead;
	
	// Read the received data
	data = *pUART_RBR;
	 
	// Calculate buffer index
	tmphead = (UART_RxHead + 1);
	if(tmphead>=UART_RX_BUFFER_SIZE) tmphead = 0;
	UART_RxHead = tmphead; 		// store new index
	UART_RxBuf[tmphead] = data; // store received data in buffer
	
	rxcnt++;
	if(rxcnt>0x3f) checkflag = YES;
	if((rxcnt>(UART_RX_BUFFER_SIZE-20))&&(xoffsent==NO)&&(enablexonoff==YES)) {
		// If tx buffer is empty then transmit xoff else signal tx to send xoff
	    sendxonxoff = YES;
	    xoffsent = YES;
		//  Fire TX interrupt
		*pUART_IER |= ETBEI;		
		}
	// Does the RX buffer flag need clearing?
	
//	if ( tmphead == UART_RxTail )
//		{
//		/* ERROR! Receive buffer overflow */
//		}
}


